#####################################################################
# Copyright (C) 1988,1989
# Fourgen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../all.4gm/i_chart.4gs/header.4gl  1.8  Delta: 8/31/90
#####################################################################

globals "globals.4gl"

#######################################################################
function llh_input()
# returning -1 if tab pressed, 0 otherwise
#######################################################################
#
    define
        dup smallint

    let dup = false
    input p_xchrtr.* without defaults from s_chrtr.*

        on key (control-i)
            return -1
        on key (control-e)
            call fg_err(0)
        on key (control-w)
            call fg_help(1)
        on key (control-f)
            # add/update user defined fields...
            if m_xchrtr.acct_no is null
              then call al_addlz("stxchrtr",p_xchrtr.acct_no,"","")
              else call al_addlz("stxchrtr",m_xchrtr.acct_no,"","")
            end if

        before field acct_no
            if not dup and activity()
            then
                next field acct_desc
            end if
        after field acct_no
            if chkdup()
            then
                let dup = true
                next field acct_no
            else
                let dup = false
                call setype()
            end if
        before field incr_with_crdt
            if activity()
            then
                next field acct_no
            end if
        after field incr_with_crdt
            if p_xchrtr.incr_with_crdt is null
            then
                call setype()
            end if

    end input
    return 0
end function
# llh_input()

#######################################################################
function llh_chkinput()
#   returns true/false based upon good/bad data in screen
#######################################################################
#   this function is called upon exiting the input statement.
#   the "after input" clause in the input statement can't call
#   this because a tab (control-i) needs to run this function also.
#
    if chkdup()
    then
        return false
    end if
    call setype()
    if mustfill(p_xchrtr.acct_no,"Account Number")
        then return false end if
    if mustfill(p_xchrtr.acct_desc,"Account Description")
        then return false end if
    if mustfill(p_xchrtr.incr_with_crdt,"Increase w/Credit")
        then return false end if

    return true
end function
# llh_chkinput()

#######################################################################
function llh_display()
#######################################################################
#   - display all p_* variables onto the screen
#
    display p_xchrtr.* to s_chrtr.*
end function
# llh_display()

#######################################################################
function llh_p_prep()
#######################################################################
#   - this function creates the p_* record from the m_* record
#
    let p_xchrtr.acct_no = m_xchrtr.acct_no
    let p_xchrtr.acct_type = m_xchrtr.acct_type
    let p_xchrtr.acct_desc = m_xchrtr.acct_desc
    let p_xchrtr.subtotal_group = m_xchrtr.subtotal_group
    let p_xchrtr.incr_with_crdt = m_xchrtr.incr_with_crdt
end function
# llh_p_prep()

#######################################################################
function llh_m_prep()
#######################################################################
#   - this function creates the m_* record from the p_* record
#
    # change key to user defined fields
    # if key was changed during input
    if   m_xchrtr.acct_no is not null
     and m_xchrtr.acct_no != p_xchrtr.acct_no
    then
        call al_addlnk("stxchrtr",m_xchrtr.acct_no,p_xchrtr.acct_no)
    end if
    let m_xchrtr.acct_no = p_xchrtr.acct_no
    let m_xchrtr.acct_type = p_xchrtr.acct_type
    let m_xchrtr.acct_desc = p_xchrtr.acct_desc
    let m_xchrtr.subtotal_group = p_xchrtr.subtotal_group
    let m_xchrtr.incr_with_crdt = p_xchrtr.incr_with_crdt
end function
# llh_m_prep()

#######################################################################
function llh_read(ring_rowid)
#######################################################################
#   - this function reads in all of the p_* variables from disk.
#   - it does not use a cursor, so it doesn't lock the row.
#
    define
        ring_rowid  integer     # rowid of record to read
    whenever error continue
    select * into m_xchrtr.* from stxchrtr where rowid = ring_rowid
    whenever error call error_handler
    if sqlca.sqlcode != 0
    then
        return
    end if
    call llh_p_prep()
    return
end function
# llh_read()

#######################################################################
function llh_delete(ring_rowid)
#######################################################################
#
    define
        ring_rowid  integer     # rowid passed
    delete from stxchrtr
        where rowid = ring_rowid
    if sqlca.sqlcode = 0
    then
        # delete user defined fields
        call al_addld("stxchrtr",p_xchrtr.acct_no)
    end if
end function
# llh_delete()

#######################################################################
function llh_add()
#######################################################################
#   - insert a new reference record, and all n-lines associated with it.
#
    whenever error continue
    insert into stxchrtr values(m_xchrtr.*)
    whenever error call error_handler
    let p_rowid = sqlca.sqlerrd[6]
end function
# llh_add()

#######################################################################
function llh_update(ring_rowid)
#######################################################################
#
    define
        ring_rowid  integer     # rowid passed
    update stxchrtr set stxchrtr.* = m_xchrtr.*
        where rowid = ring_rowid
end function
# llh_update()

#######################################################################
function chkdup()
# returning true/false   dup/no dup
#######################################################################
# - this function checks for a duplicate acct #
    define
        counter     smallint
    # if updating, don't check for dups because you will always find
    # yourself as a dup.  (no acct_no changes are allowed upon update anyway)
    select count(*) into counter from stxchrtr where
        acct_no = p_xchrtr.acct_no and rowid != p_rowid
    if counter > 0
    then
        call fg_err(1) # Duplicate Account Number
        return true
    end if
    return false
end function
# chkdup()

#######################################################################
function setype()
#######################################################################
# - this function takes the account number and figures the acct type.
    if p_xchrtr.acct_no < m_xcntrc.fixed_asset then
        call setfld("CURRENT ASSETS","A","1","N") return end if
    if p_xchrtr.acct_no < m_xcntrc.curr_liab then
        call setfld("FIXED ASSETS","A","2","N") return end if
    if p_xchrtr.acct_no < m_xcntrc.long_term_liab then
        call setfld("CUR LIABILITIES","B","3","Y") return end if
    if p_xchrtr.acct_no < m_xcntrc.capital then
        call setfld("L/T LIABILITIES","B","4","Y") return end if
    if p_xchrtr.acct_no < m_xcntrc.income then
        call setfld("CAPITAL","C","5","Y") return end if
    if p_xchrtr.acct_no < m_xcntrc.cost_goods then
        call setfld("INCOME","D","6","Y") return end if
    if p_xchrtr.acct_no < m_xcntrc.expense then
        call setfld("COST OF GOODS","E","7","N") return end if
    call setfld("EXPENSES","F","8","N")
end function
# setype()

#######################################################################
function setfld(acct_type,acct_cat,processing_seq,incr_with_crdt)
#######################################################################
#
    define
        acct_type   like stxchrtr.acct_type,
        acct_cat    like stxchrtr.acct_cat,
        processing_seq  like stxchrtr.processing_seq,
        incr_with_crdt  like stxchrtr.incr_with_crdt

    let p_xchrtr.acct_type = acct_type
    display by name p_xchrtr.acct_type
    let m_xchrtr.acct_cat = acct_cat
    let m_xchrtr.processing_seq = processing_seq
    if p_xchrtr.incr_with_crdt is null
    then
        let p_xchrtr.incr_with_crdt = incr_with_crdt
        display by name p_xchrtr.incr_with_crdt
    end if
end function
# setfld()

#######################################################################
function activity()
# returning true/false based on activity/no_activity posted.
#######################################################################
#
    define
        counter smallint

    # if account number is null, return false
    if p_xchrtr.acct_no is null then return false end if

    # check for activity in history file
    select count(*) into counter from stxchrtd
            where acct_no = p_xchrtr.acct_no
    if counter > 0
    then
        return true
    end if
    # check for activity in activity file
    select count(*) into counter from stgactvd
            where acct_no = p_xchrtr.acct_no
    if counter > 0
    then
        return true
    end if
    # can't find any activity
    return false
end function
# activity()
