#####################################################################
# Copyright (C) 1988,1989
# Fourgen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../lib.4gm/all.4gs/al_addlz.4gl  1.4  Delta: 12/11/89
#####################################################################

database standard

############################################################
function al_addlinit()
############################################################
# This function initializes the addl zoom routine.  It is
# designed to be called upon init, and it's job is to
# prepare the sql statement and declare the cursor so it
# doesn't have to be done everytime al_addlz is called.
#
    prepare get_head from
        "select * from stxaddlr where filename = ?"
    declare head_curs cursor for get_head

    prepare get_detl from
        "select * from stxaddld where filename = ? and record_key = ?"
    declare detl_curs cursor for get_detl

    prepare insert_addl from
        "insert into stxaddld values (?, ?, ?, ?)"

    prepare update_key from
      "update stxaddld set record_key = ? where filename = ? and record_key = ?"

    prepare del_detl from
        "delete from stxaddld where filename = ? and record_key = ?"

end function
# al_addlinit()

############################################################
function al_addlnk(filename, old_key, new_key)
############################################################
# This function changes the record_key from old_key
# to new_key.  Needed when the key changes in the table.
#
    define
        filename like stxaddld.filename,
        old_key like stxaddld.record_key,
        new_key like stxaddld.record_key

    if new_key is not null
    then
        whenever error continue
        execute update_key using new_key, filename, old_key
        whenever error call error_handler
    end if
    return
end function
# al_addlnk()

############################################################
function al_addld(filename, record_key)
############################################################
# This function deletes all of the data in the stxaddld
# table that matches the filename, record_key criteria.
#
    define
        filename like stxaddld.filename,
        record_key like stxaddld.record_key

    if record_key is not null
    then
        whenever error continue
        execute del_detl using filename, record_key
        whenever error call error_handler
    end if
    return
end function
# al_addld()

############################################################
function al_addlz(filename, key1, key2, key3)
############################################################
# This function displays a zoom window to the additional
# fields data.  It allows the user to define additional
# fields and enter data into them.
# The 3 seperate keys are clipped, and concatenated together
# to build the record_key.  They are passed as 3 seperate
# arguments so the calling program doesn't have to have
# a string just for that purpose
#
    define
        filename like stxaddld.filename,      # argument passed
        key1 char(20),                        # argument
        key2 char(20),                        # argument
        key3 char(20),                        # argument
        record_key like stxaddld.record_key,  # record key
        m_xaddlr record like stxaddlr.*,      # record like addlr table
        m_xaddld record like stxaddld.*,      # record like addld table
        p_addl array[50] of record            # like the screen array
            line_no smallint,
            field_label like stxaddlr.field_label,
            data like stxaddld.data
        end record,
        is_add smallint,                      # boolean: is this add or update?
        ok_update smallint,                   # boolean: ok to update label?
        n smallint                            # generic counter

    # Build the record key
    let record_key = key1 clipped, key2 clipped, key3 clipped

    # Don't allow the entry of a null key.
    if record_key = " " then return end if

    open window al_addl at 6,9
        with form "../../lib.4gm/all.4gs/al_addlz"
            attribute (border, blue)

    # insert line numbers
    for n = 1 to 50
        let p_addl[n].line_no = n
        let p_addl[n].field_label = null
        let p_addl[n].data = null
    end for

    # display the table, and key
    display "Table: ", filename at 15, 2
    display "Key: ", record_key at 15, 31

    # insert field names
    open head_curs using filename
    foreach head_curs into m_xaddlr.*
        let n = m_xaddlr.line_no
        let p_addl[n].field_label = m_xaddlr.field_label
    end foreach

    # insert data
    open detl_curs using filename, record_key
    foreach detl_curs into m_xaddld.*
        let n = m_xaddld.line_no
        let p_addl[n].data = m_xaddld.data
    end foreach

    call menu_line()
    display "Update:  [ESC] to Store  [DEL] to Cancel" at 1,2
    display "[DOWN ARROW] or [UP ARROW] to Scroll,  [F3] or [F4] to Page" at 2,2

    # setup for the input array
    call set_count(50)

    input array p_addl without defaults from s_addlz.*
        before field line_no
            next field data
        before field field_label
            if ok_update = false
              then next field data
            end if
            let n = arr_curr()
            if p_addl[n].field_label is null
              then let is_add = true
              else let is_add = false
            end if
        before field data
            if ok_update
              then let ok_update = false
            end if
        after field field_label
            whenever error continue
            if is_add
            then
                insert into stxaddlr values(filename,
                  p_addl[n].line_no, p_addl[n].field_label)
            else
                update stxaddlr set field_label = p_addl[n].field_label
                  where filename = @filename
                    and line_no = p_addl[n].line_no
            end if
            whenever error call error_handler
        on key(control-f)
            if ok_update
              then let ok_update = false
              else let ok_update = true
            end if
            next field field_label
    end input

    # process [ESC] or [DEL]
    if int_flag != 0
    then
        # delete pressed
        let int_flag = 0
    else
        # store pressed (esc)
        call al_addld(filename, record_key)
        for n = 1 to 50
            if p_addl[n].data is not null
            then
                whenever error continue
                execute insert_addl using filename, record_key,
                  p_addl[n].line_no, p_addl[n].data
                whenever error call error_handler
            end if
        end for
    end if

    close window al_addl
    return
end function
# al_addlz()
