########################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../lib.4gm/all.4gs/ap_last.4gl  1.3  Delta: 9/4/90
########################################################################

globals "ap_glob.4gl"

########################################################################
function ap_last(post_or_check, doc_type, doc_amount, vend_code)
# returns true/false based on pass/fail of the routine
########################################################################
# This function is passed a post-or-check code ("only CHECK"
# is valid), a document type (only "CD" is valid), a check amount
# and a vendor code.
# This routine checks if the document balances, and if it does, it then
# updates the vendor balance in the vendor table.  The doc_amount
# that is passed is the amount of the check.  It is only used for
# doc_types of "CD" (cash disbursments) and determines whether the
# check amount balances with all of the distributions or not.
#
# status and description variables returned in the post_ap record:
#  0 - Successful
#  1 - fg_error("lib_all","ap_last", 1) - A/P Isn't Installed
#  3 - fg_error("lib_all","ap_last", 3) - A/P Setup Isn't Complete
# 10 - fg_error("lib_all","ap_last",10) - Document Doesn't Balance.
# 11 - fg_error("lib_all","ap_last",11) - Cannot update Vendor Balance.
#
    define
        post_or_check char(5),
        doc_type char(2),
        doc_amount like stgactvd.amount,
        vend_code like stxtranr.ref_code,
        sql_str char(150),
        quote char(1),
        stat_flag smallint

    let post_ap.status = 0
    let quote = "\""
    if post_ap.ap_installed = "N"
    then
        let post_ap.status = 1
        let post_ap.description = "A/P Isn't Installed"
        return true
    end if

    if post_ap.ap_balanced != "Y"
    then
        let post_ap.status = 3
        let post_ap.description = "A/P Setup Isn't Complete"
        return false
    end if

    if doc_type = "CD"
    then
        if post_ap.ap_accum - post_ap.disc_accum != (doc_amount * -1)
        then
            let post_ap.status = 10
            let post_ap.description = "Document Doesn't Balance"
            return false
        end if
    end if

    if post_or_check = "CHECK"
    then
        let post_ap.status = 0
        return true
    end if

    # Update the vendor's balance
    let sql_str = "update stpvendr set acct_bal = acct_bal + ?"
    if doc_type = "CD"
    then
        let sql_str = sql_str clipped, ", last_pay_date = ",
            quote, today using "mm/dd/yy", quote
    end if

    let sql_str = sql_str clipped, " where vend_code = ?"
    let stat_flag = true
    whenever error continue
    prepare upd1 from sql_str
    execute upd1 using post_ap.ap_accum, vend_code
    whenever error call error_handler
    if sqlca.sqlcode != 0
    then
        let stat_flag = false
        let sqlca.sqlcode = 0
        open window wt3_win at 10,4 with 1 rows, 60 columns
            attribute(white,border)
        display " Waiting for vendor: '",vend_code clipped,"' to become available..."
            at 1,1
        sleep 5
    end if
    let int_flag = 0
    while stat_flag = false
        whenever error continue
        execute upd1 using post_ap.ap_accum, vend_code
        whenever error call error_handler
        if sqlca.sqlcode = 0
        then
            let sqlca.sqlcode = 0
            exit while
        else
            if int_flag != 0
            then
                let post_ap.status = 11
                let post_ap.description = "Cannot Update Vendor Balance"
                return false
            end if
            let sqlca.sqlcode = 0
            sleep 5
        end if
    end while
    if stat_flag = false
    then
        close window wt3_win
    end if

    return true
end function
# ap_last()

