########################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../lib.4gm/all.4gs/ar_last.4gl  1.4  Delta: 9/4/90
########################################################################

globals "ar_glob.4gl"

########################################################################
function ar_last(post_or_check, doc_type, doc_amount, cust_code)
# returns true/false based on pass/fail of the routine
########################################################################
#  This function is passed a post-or-check code (only "CHECK"
#  is valid), a document type (only "CR" is valid), a check amount
#  and a vendor code.
#
#  For doc_types of CR, this routine checks if the document balances.
#  That is, it determines whether the check amount balances with all of
#  the distributions.
#
#  In all cases, it updates the customer balance in the customer table.
#  For CR, The doc_amount that is passed is the amount of the check.
#  For other doc_types, the doc_amount is the A/R amount.
#
# status and description variables returned in the post_ar record:
#  0 - Successful
#  1 - fg_error("lib_all","ar_last", 1) - A/R Isn't Installed
#  3 - fg_error("lib_all","ar_last", 3) - A/R Setup Isn't Complete
# 10 - fg_error("lib_all","ar_last",10) - Document Doesn't Balance.
# 11 - fg_error("lib_all","ar_last",11) - Cannot update Customer Balance.
#
    define
        post_or_check char(5),
        doc_type char(2),
        doc_amount like stgactvd.amount,
        cust_code like stxtranr.ref_code,
        sql_str char(150),
        quote char(1),
        stat_flag smallint

    let post_ar.status = 0
    let quote = "\""
    if post_ar.ar_installed = "N"
    then
        let post_ar.status = 1
        let post_ar.description = "A/R Isn't Installed"
        return true
    end if

    if post_ar.ar_balanced != "Y"
    then
        let post_ar.status = 3
        let post_ar.description = "A/R Setup Isn't Complete"
        return false
    end if

    if doc_type = "CR"
    then
        if post_ar.ar_accum - post_ar.disc_accum != (doc_amount * -1)
        then
            let post_ar.status = 10
            let post_ar.description = "Document Doesn't Balance"
            return false
        end if
    end if

    if post_or_check = "CHECK"
    then
        let post_ar.status = 0
        return true
    end if

    # Update the customer's balance
    let sql_str = "update strcustr set acct_bal = acct_bal + ?"
    if doc_type = "CR"
    then
        let sql_str = sql_str clipped, ", last_pay_date = ",
            quote, today using "mm/dd/yy", quote
    end if

    let sql_str = sql_str clipped, " where cust_code = ?"
    let stat_flag = true
    whenever error continue
    prepare upd1 from sql_str
    execute upd1 using post_ar.ar_accum, cust_code
    whenever error call error_handler
    if sqlca.sqlcode != 0
    then
        let sqlca.sqlcode = 0
        let stat_flag = false
        open window wait3_win at 10,4 with 1 rows, 60 columns
            attribute(white,border)
        display " Waiting for customer: '",cust_code clipped,"' to become available..."
            at 1,1
        sleep 5
    end if
    let int_flag = 0
    while stat_flag = false
        whenever error continue
        execute upd1 using post_ar.ar_accum, cust_code
        whenever error call error_handler
        if sqlca.sqlcode = 0
        then
            exit while
        else
            let sqlca.sqlcode = 0
            if int_flag != 0
            then
                let post_ar.status = 11
                let post_ar.description = "Cannot Update Customer Balance"
                return false
            end if
            sleep 5
        end if
    end while
    if stat_flag = false
    then
        close window wait3_win
    end if

    return true
end function
# ar_last()

