########################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../lib.4gm/all.4gs/ar_post.4gl  1.6  Delta: 9/4/90
########################################################################

globals "ar_glob.4gl"

########################################################################

function ar_post(post_or_check, orig_journal, doc_no,
                 post_no, post_date, doc_date, ref_code, doc_desc,
                 inv_chk_no, doc_type,
                 act_type, inv_doc_no, amount,
                 disc_date, disc_bal, due_date, ar_acct_no, ar_department,
                 po_no, po_date)
# returns true/false based on pass/fail of the routine
########################################################################
#  This routine takes the A/R transaction data as arguments,
#  and either posts to A/R or checks for an ok posting.
#  It is designed to be run in "CHECK" mode during the edit list phase,
#  and in "POST" mode during the posting phase.
#  It modifies elements in the global record
#  post_ar defined in ar_glob.4gl in this directory.
#  All routines that post into the 4gen accounts receivable module should
#  do so via this routine.
#
#  Invoices call this routine once per invoice.
#
#  Credit/Debit Memos call this routine once per memo.
#
#  Checks call this routine once (or twice if paying with a discount)
#    for each item that is paid with that check.
#
#  This routine uses prepares/declares/and executes that build the
#    sql query from a string because it needs to be able to compile
#    without any A/R files installed.
#
#  Data elements passed:
#    post_or_check char(5),  "POST" or "CHECK"  tells ar_post to check
#                            the posting (for edit lists), or post it.
#    orig_journal char(2),   original journal (AR, OE, etc...)
#    doc_no integer,         document number (sequential and congruent for
#                            each document within a journal)
#    post_no integer,        posting number (post only)
#    post_date date,         posting date (post only)
#    doc_date date,          document entry date
#    ref_code char(6),       customer code
#    doc_desc char(30),      document description
#    inv_chk_no char(10),    invoice# or check# (freeform - not used as a key)
#    doc_type char(2),       Document Types:
#                            IN = Invoice, CM = Credit Memo,
#                            DM = Debit Memo, CR = Check, FC = Finance Charge
#    act_type char(1),       Activity types:
#                            A = Transaction that affects the A/R balance,
#                            D = Transaction that affects the Discount balance
#    inv_doc_no integer,     Invoice Document Number (0 for "on account")
#    amount like stgactvd.amount,   Signed amount (+increases, -decreases A/R balances)
#  -   -   -   -   -  Fields below are only filled if doc_type = "IN" or "FC"
#    disc_date date,         Date the discount becomes inactive
#    disc_bal like stgactvd.amount, Initial discount amount
#    due_date date,          Date invoice is due
#    ar_acct_no integer,     A/R account that invoice was posted into
#                            (so credits against the invoice will post into
#                            the same A/R account)
#    ar_department char(3),  ar department that invoice was posted into
#    po_no char(10),         purchase order number
#    po_date date            date purchase order was written
#
# status and description variables returned in the post_ar record:
# 0 - Successful
# 1 - fg_error("lib_all","ar_post",1) - A/R isn't installed
#     - return true
# 2 - fg_error("lib_all","ar_post",2) - Cannot Read A/R Control Table.
#     - return false
# 3 - fg_error("lib_all","ar_post",3) - A/R Setup Isn't Complete
#     - (ar_balanced isn't "Y" in strcntrc)-return false
# 4 - fg_error("lib_all","ar_post",4) - Document Number out of Sequence
#     - (warning condition only) - continue
# 5 - fg_error("lib_all","ar_post",5) - Duplicate Document Number Exists
#     - (error condition) - return false
# 6 - fg_error("lib_all","ar_post",6) - Cannot Insert a New Document
#     - (sql insert command failed) - return false
# 7 - fg_error("lib_all","ar_post",7) - Cannot Update Customer On-Acct Balance.
#     - return false
# 8 - fg_error("lib_all","ar_post",8) - Cannot Update Open Item Balance.
#     - return false
#
# What this process does:
# Check to see if A/R is installed.  If not, just return true...
# If last doc no isn't this doc no (first in a group)
#     if last doc no isnt this doc no - 1, then set warning #4
#     call trx_post - this (possibly) posts a new document into
#         stxtranr (top level)
#         trx_post returns:
#         0 = if "CHECK" or if "POST" and posted ok
#         1 = insert failed (post only)
#         2 = dup found
#     if "CHECK", check for duplicate strtranr
#     if "POST", insert into strtranr
# if (first in group)
# if "CHECK", then return
# if "POST" then:
#     if invoice type
#     then add open item into stropend
#     else
#         if inv_doc_no != 0
#         then update the open item
#         else update the on-account amount
#     end if
# if
#
# This process is designed to be run in conjunction with the ar_last()
# library function that posts the customer balance to the customer file and
# also checks to make sure the entire transaction balances.
#
    define
        post_or_check char(5),
        orig_journal like stxtranr.orig_journal,
        doc_no like stxtranr.doc_no,
        post_no like stxtranr.post_no,
        post_date like stxtranr.post_date,
        doc_date like stxtranr.doc_date,
        ref_code like stxtranr.ref_code,
        doc_desc like stxtranr.doc_desc,
        inv_chk_no char(10),
        doc_type char(2),
        act_type char(1),
        inv_doc_no integer,
        amount like stgactvd.amount,
        disc_date date,
        disc_bal like stgactvd.amount,
        due_date date,
        ar_acct_no integer,
        ar_department char(3),
        po_no char(10),
        po_date date,
        sql_str char(150),
        stat_flag smallint,
        doc_status smallint

    let post_ar.status = 0
    # First time... fill post_ar.ar_installed and post_ar.ar_balanced
    if post_ar.ar_installed is null
    then
        whenever error continue
        select tabname into post_ar.description
           from systables where tabname = "strcntrc"
        whenever error call error_handler
        if sqlca.sqlcode != 0
        then
            let sqlca.sqlcode = 0
            let post_ar.ar_installed = "N"
        else
            let post_ar.ar_installed = "Y"
        end if
        if post_ar.ar_installed = "Y"
        then
            let sql_str = "select ar_balanced from strcntrc"
            whenever error continue
            prepare sel1 from sql_str
            declare selec1 cursor for sel1
            open selec1
            foreach selec1 into post_ar.ar_balanced
            end foreach
            close selec1
            whenever error call error_handler
            if sqlca.sqlcode != 0
            then
                let sqlca.sqlcode = 0
                let post_ar.status = 2
                let post_ar.description = "Cannot Read A/R Control Table"
                return false
            end if
        end if
    end if

    if post_ar.ar_installed = "N"
    then
        let post_ar.status = 1
        let post_ar.description = "A/R Isn't Installed"
        return true
    end if

    if post_ar.ar_balanced != "Y"
    then
        let post_ar.status = 3
        let post_ar.description = "A/R Setup Isn't Complete"
        return false
    end if

    if doc_no = post_ar.last_doc_no
    then else
        # first of a group
        if post_ar.last_doc_no is not null
           and post_ar.last_doc_no != doc_no - 1
           and post_or_check = "POST"
        then
            let post_ar.status = 4
            let post_ar.description = "Document Number out of Sequence"
        end if
        let post_ar.last_doc_no = doc_no
        let post_ar.ar_accum = 0
        let post_ar.disc_accum = 0
        call trx_post(post_or_check, orig_journal, doc_no,
                 post_no, post_date, doc_date, ref_code, doc_desc)
            returning doc_status
        if doc_status = 1
        then
            let post_ar.status = 6
            let post_ar.description = "Cannot Insert a New Document.",
                                      " (stxtranr)"
            return false
        end if
        if doc_status = 2
        then
            let post_ar.status = 5
            let post_ar.description = "Duplicate Document Number Exists",
                                      " (stxtranr)"
            return false
        end if
        if post_or_check = "POST"
        then
            let sql_str = "select count(*) from strtranr",
                " where orig_journal = ? and doc_no = ?"
            whenever error continue
            prepare cnt1 from sql_str
            declare count1 cursor for cnt1
            open count1 using orig_journal, doc_no
            foreach count1 into post_no
            end foreach
            close count1
            whenever error call error_handler
            if post_no > 0
            then
                let post_ar.status = 5
                let post_ar.description = "Duplicate Document Number Exists.",
                                          " (strtranr)"
                return false
            end if

            # ok to insert...
            whenever error continue
            let sql_str = "insert into strtranr values (?, ?, ?, ?)"
            whenever error continue
            prepare ins1 from sql_str
            execute ins1 using orig_journal, doc_no, inv_chk_no, doc_type
            whenever error call error_handler
            if sqlca.sqlcode != 0
            then
                let sqlca.sqlcode = 0
                let post_ar.status = 6
                let post_ar.description = "Cannot Insert a New Document.",
                                          " (strtranr)"
                return false
            end if
        end if   # POST
    end if   # first document in batch
    # Accumulators...
    # the ar_accum for invoices is the invoice amount
    # for checks, it is the ar amount
    # and disc_accum (non-invoices only) is the discount amount
    if act_type = "A"
    then
        # amount is an A/R amount
        let post_ar.ar_accum = post_ar.ar_accum + amount
    else
        # amount is a discount amount
        let post_ar.disc_accum = post_ar.disc_accum + amount
    end if

    if post_or_check = "CHECK"
    then
        let post_ar.status = 0
        return true
    end if

    # POST type...
    # insert the stractvd row
    let sql_str = "insert into stractvd values (?, ?, ?, ?, ?)"
    whenever error continue
    prepare ins2 from sql_str
    execute ins2 using orig_journal, doc_no, act_type, inv_doc_no, amount
    whenever error call error_handler
    if sqlca.sqlcode != 0
    then
        let sqlca.sqlcode = 0
        let post_ar.status = 6
        let post_ar.description = "Cannot Insert a New Document",
                                  " (stractvd)"
        return false
    end if

    # add to the open item table if an invoice
    if doc_type = "IN" or doc_type = "FC"
    then
        let sql_str = "insert into stropend values ",
            "(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)"
        whenever error continue
        prepare ins3 from sql_str
        execute ins3 using
            ref_code, inv_chk_no, doc_no, doc_desc, doc_date, amount,
            disc_bal, amount, disc_bal, due_date, disc_date, ar_acct_no,
            ar_department, po_no, po_date, doc_type
        whenever error call error_handler
        if sqlca.sqlcode != 0
        then
            let sqlca.sqlcode = 0
            let post_ar.status = 6
            let post_ar.description = "Cannot Insert a New Document",
                                      " (stropend)"
            return false
        end if
    else
        # not an invoice - an entry that affects (adjusts) the open item
        if inv_doc_no = 0
        then
            # on account - update customer on account amount in strcustr
            let sql_str = "update strcustr set on_acct_amt = on_acct_amt + ?",
                          " where cust_code = ?"
            let stat_flag = true
            whenever error continue
            prepare upd1 from sql_str
            execute upd1 using amount, ref_code
            whenever error call error_handler
            if sqlca.sqlcode != 0
            then
                let sqlca.sqlcode = 0
                let stat_flag = false
                open window wait1_win at 10,4 with 1 rows, 60 columns
                    attribute(white,border)
                display " Waiting for customer: '",ref_code clipped,"' to become available..."
                    at 1,1
                sleep 5
            end if
            let int_flag = 0
            while stat_flag = false
                whenever error continue
                execute upd1 using amount, ref_code
                whenever error call error_handler
                if sqlca.sqlcode = 0
                then
                    exit while
                else
                    let sqlca.sqlcode = 0
                    if int_flag != 0
                    then
                        let post_ar.status = 7
                        let post_ar.description =
                            "Cannot Update Customer On-Account Balance"
                        return false
                    end if
                    sleep 5
                end if
            end while
            if stat_flag = false
            then
                close window wait1_win
            end if
        else
            # not on account - update the open item in stropend
            if act_type = "A"
            then
                # update A/R (invoice) balance
                let sql_str = "update stropend set balance = balance + ?",
                              " where doc_no = ?"
            else
                # update Discount balance
                let sql_str = "update stropend set disc_bal = disc_bal + ?",
                              " where doc_no = ?"
            end if
            let stat_flag = true
            whenever error continue
            prepare upd2 from sql_str
            execute upd2 using amount, inv_doc_no
            whenever error call error_handler
            if sqlca.sqlcode != 0
            then
                let sqlca.sqlcode = 0
                let stat_flag = false
                open window wait2_win at 10,4 with 1 rows, 60 columns
                    attribute(white,border)
                display " Waiting for open item doc. no: '",
                    inv_doc_no using "<<<<<<<<<" ,
                    "' to become available..."
                    at 1,1
                sleep 5
            end if
            let int_flag = 0
            while stat_flag = false
                whenever error continue
                execute upd2 using amount, inv_doc_no
                whenever error call error_handler
                if sqlca.sqlcode = 0
                then
                    exit while
                else
                    let sqlca.sqlcode = 0
                    if int_flag != 0
                    then
                        let post_ar.status = 8
                        let post_ar.description =
                            "Cannot Update Open Item Balance"
                        return false
                    end if
                    sleep 5
                end if
            end while
            if stat_flag = false
            then
                close window wait2_win
            end if
        end if    # on account (inv_doc_no = 0)
    end if   # invoice type "IN"
    return true
end function
# ar_post()

