########################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../lib.4gm/all.4gs/fg_err.4gl  1.5  Delta: 9/4/90
########################################################################

globals "globals.4gl"

############################################################
function fg_err(err_num)
# displays an error message
############################################################
# This function is passed an error number and displays an
# error message to the user.  The user is presented the 
# option of being shown additional information from error
# table, stxerorh.  If this option is selected the user also
# has the option of updating the error information.
#

    define
        err_mod     char(8),    # module name
        err_prog    char(8),    # program name within the module
        err_num     smallint,   # error number within the program
        scratch     char(80),   # scratch string area
        c           char(1)     # entry char

    call get_mod() returning err_mod, err_prog

    if err_num = 0
    then
        call err_choose(err_mod,err_prog)
        return
    end if

    open window errwin at 22, 8 with 2 rows, 66 columns
        attribute (border, white)

    display " \007Error:" at 1,1 attribute(white)

    whenever error continue
    select err_line into scratch from stxerorh
        where stxerorh.err_module=err_mod
          and stxerorh.err_program=err_prog
          and stxerorh.err_number=err_num
    whenever error call error_handler
    if sqlca.sqlcode != 0
    then
        let sqlca.sqlcode = 0
        let scratch = "Cannot find '",err_mod clipped,".",err_prog clipped,
            ".",err_num using "<<<<" clipped,"' in error table."
        #let err_mod = "lib"
        #let err_prog = "err"
        #let err_num = 1
         insert into stxerorh (err_module, err_program, err_number, err_line)
             values (err_mod, err_prog, err_num, scratch)
    end if

    display " ",scratch clipped," " at 1,10 attribute(reverse)
    prompt
        " Press [ENTER] to continue, or [CTRL]-[Z] for more information: "
        for char c
        on key(control-z)
            call err_info(err_mod, err_prog, err_num, scratch)
    end prompt

    close window errwin
    return
end function
# fg_err()

############################################################
function err_choose(err_mod,err_prog)
# displays error messages for selection
############################################################
# This function is called by function fg_err.  Using the 
# passed parameters err_mod and err_prog it displays errors
# in a window associated with the module and program names
# that have been passed.  The user may then select an error
# message to obtain more about.
#
    define
        err_mod     char(8),    # module name
        err_prog    char(8),    # program name within the module
        err_choose array[30] of record
            module  like stxerorh.err_module,
            prog    like stxerorh.err_program,
            number  like stxerorh.err_number,
            err_line like stxerorh.err_line
        end record,
        counter  smallint,
        pcurr    smallint,
        scurr    smallint

    let pcurr = arr_curr()
    let scurr = scr_line()

    select count(*) into counter
                    from stxerorh where
                         stxerorh.err_module = err_mod
                    and  stxerorh.err_program = err_prog

    if counter = 0
    then
        return
    end if

    open window errchoose at 3, 6
        with form "../../lib.4gm/all.4gs/errbrws"
            attribute (border, blue)

    display " Select an error and press [ESC] to choose" at 2,1

    call menu_line()

    let counter = 1

    declare err_list cursor for
         select * from stxerorh where
                stxerorh.err_module = err_mod
           and  stxerorh.err_program = err_prog
           order by  err_number

    foreach err_list into err_choose[counter].*
        let counter = counter + 1
        if counter > 30
        then
            exit foreach
        end if
    end foreach

    call set_count(counter -1)

    display array err_choose to s_xerorh.*

    let counter = arr_curr()

    if int_flag != 0
    then
        let int_flag = 0
        close window errchoose
        return
    end if

    call err_info(err_choose[counter].module,
                  err_choose[counter].prog,
                  err_choose[counter].number,
                  err_choose[counter].err_line)

    close window errchoose

    return

end function
#err_choose

############################################################
function err_info(err_mod,err_prog,err_num,scratch)
# permits viewing and updating the additional error info
############################################################
# This function is passed the name of the module, the name of
# the program, the error number and the error message.  It 
# presents the user with the options of viewing or updating
# the the additional error information.  It is called by the
# functions fg_err and err_choose.
#
    define
        err_mod     char(8),    # module name
        err_prog    char(8),    # program name within the module
        err_num     smallint,   # error number within the program
        ere_mod     char(8),    # module name (entered)
        ere_prog    char(8),    # program name within the module (entered)
        ere_num     smallint,   # error number within the program (entered)
        scratch     char(50),   # scratch string area
        err_code    char(20),   # error code "mod.prog.num"
        tab_pressed smallint,   # tab key pressed
        n           smallint,   # working number
        o           smallint,   # working number
        p           smallint,   # working number
        m_xerord    record like stxerord.*, # record like err file
        p_err_line like stxerorh.err_line,
        q_err_line like stxerorh.err_line,
        p_xerord array[25] of record   # record like the detail screen
            err_text like stxerord.err_text
            end record,
        q_xerord array[25] of record   # record like the detail screen
            err_text like stxerord.err_text
            end record,
        r_xerord array[25] of record   # record like the detail screen
            err_text like stxerord.err_text
            end record,
        s_xerord array[25] of record   # record like the detail screen
            err_text like stxerord.err_text
            end record,
        ext_status  smallint,           # exit input status
        rec1_cnt    smallint,           # record count
        rec2_cnt    smallint,           # record count
        rec3_cnt    smallint,           # record count
        rec4_cnt    smallint            # record count

    open window errtxtwin at 4, 8 with 16 rows, 66 columns
        attribute (border, white)

    open form errtxt from "../../lib.4gm/all.4gs/errtxt"
    display form errtxt attribute(blue)

    call menu_line()

    let err_code = err_mod clipped,".",err_prog clipped,
            ".",err_num using "<<<<" clipped

    let q_err_line = scratch
    let p_err_line = scratch

    display scratch to s_xerorh.err_txt attribute(red)

    if err_mod = "lib" and err_prog = "err"
    then
        # do nothing
    else
        whenever error continue
        declare d_err cursor for
            select * from stxerord where
                stxerord.err_module = err_mod
           and  stxerord.err_program = err_prog
           and  stxerord.err_number = err_num
            order by stxerord.a_b, stxerord.line_no
        open d_err
        let n = 0
        let o = 0
        let p = 0
        foreach d_err into m_xerord.*
            let n = n + 1
            if (o > 25) or (p > 25)
            then
                error " Error: Too many rows in stxerord to read. "
                let rec1_cnt = 0
                close d_err
                close window errtxtwin
                return
            end if
            if m_xerord.a_b = "a"
            then
                let o = o + 1
                let p_xerord[o].err_text = m_xerord.err_text
                let r_xerord[o].err_text = m_xerord.err_text
                if o < 5
                then
                    display p_xerord[o].* to a_xerord[o].* attribute(red)
                end if
            else
                let p = p + 1
                let q_xerord[p].err_text = m_xerord.err_text
                let s_xerord[p].err_text = m_xerord.err_text
                if p < 5
                then
                    display q_xerord[p].* to b_xerord[p].* attribute(red)
                end if
            end if
        end foreach
        let rec1_cnt = o
        let rec2_cnt = p
        close d_err
        whenever error call error_handler
    end if

    menu " Action"
        command key (control-i,t,v)
            "View" " View document detail lines"
            # call ring_clear()
            display " [DOWN ARROW] or [UP ARROW] to Scroll, ",
                "[F3] or [F4] to Page" at 2,1
            if rec1_cnt > 0
            then
                display " Scroll:  [TAB] to solutions, [DEL] or [ESC] ",
                    "to Quit" at 1,1
                let tab_pressed = false
                call set_count(rec1_cnt)
                display array p_xerord to a_xerord.*
                    on key(control-i)
                        let tab_pressed = true
                        exit display
                end display
                if not tab_pressed
                then
                    let int_flag = 0
                    continue menu
                end if
            end if
            if rec2_cnt > 0
            then
                display "" at 1,1
                display " Scroll:  [TAB], [DEL], or [ESC]  to Quit"
                    at 1,1
                call set_count(rec2_cnt)
                display array q_xerord to b_xerord.*
                    on key(control-i)
                        exit display
                end display
            end if
            let int_flag = 0

        command key (control-c)
            # copy error text from somewhere else...
            let int_flag = 0
            prompt " COPY: Enter module to copy from: " for ere_mod
            prompt " COPY: Enter program name: " for ere_prog
            prompt " COPY: Enter error number: " for ere_num
            if int_flag != 0
            then
                continue menu
            end if
            for n = 1 to 25
                initialize p_xerord[n].* to null
                initialize q_xerord[n].* to null
                initialize r_xerord[n].* to null
                initialize s_xerord[n].* to null
                if n < 5
                then
                    display p_xerord[n].* to a_xerord[n].*
                    display q_xerord[n].* to b_xerord[n].*
                end if
            end for
            whenever error continue
            declare e_err cursor for
                select * from stxerord where
                    stxerord.err_module = ere_mod
               and  stxerord.err_program = ere_prog
               and  stxerord.err_number = ere_num
                order by stxerord.a_b, stxerord.line_no
            open e_err
            let n = 0
            let o = 0
            let p = 0
            foreach e_err into m_xerord.*
                let n = n + 1
                if (o > 25) or (p > 25)
                then
                    error " Error: Too many rows in stxerord to read. "
                    let rec1_cnt = 0
                    close e_err
                    close window errtxtwin
                    return
                end if
                if m_xerord.a_b = "a"
                then
                    let o = o + 1
                    let p_xerord[o].err_text = m_xerord.err_text
                    let r_xerord[o].err_text = m_xerord.err_text
                    if o < 5
                    then
                        display p_xerord[o].* to a_xerord[o].* attribute(red)
                    end if
                else
                    let p = p + 1
                    let q_xerord[p].err_text = m_xerord.err_text
                    let s_xerord[p].err_text = m_xerord.err_text
                    if p < 5
                    then
                        display q_xerord[p].* to b_xerord[p].* attribute(red)
                    end if
                end if
            end foreach
            let rec1_cnt = o
            let rec2_cnt = p
            close e_err
            whenever error call error_handler

        command key (u) "Update" " Update error text"
            display " Update:  [ESC] to Store  [DEL] to Cancel  ",
                    "[TAB] Next Window" at 1,1
            display " Enter changes into form" at 2,1
            let rec3_cnt = rec1_cnt
            let rec4_cnt = rec2_cnt
            while true
                let ext_status = 1          # exit is a SAVE
                input p_err_line without defaults from s_xerorh.err_txt
                    on key (control-i)
                        let ext_status = 0  # exit is a TAB
                        exit input
                    on key (interrupt)
                        let ext_status = 2  # exit is an INTERRUPT
                        let int_flag = 0
                        exit input
                end input
                if ext_status != 0      # exit is not a TAB
                then
                    exit while
                end if
                let ext_status = 1          # exit is a SAVE
                call set_count(rec3_cnt)
                input array p_xerord without defaults from a_xerord.*
                    on key (control-i)
                        let ext_status = 0  # exit is a TAB
                        exit input
                    on key (interrupt)
                        let ext_status = 2  # exit is an INTERRUPT
                        let int_flag = 0
                        exit input
                end input
                if ext_status != 2      # exit is not an INTERRUPT
                then
                    let rec3_cnt = arr_count()
                end if
                if ext_status != 0      # exit is not a TAB
                then
                    exit while
                end if
                let ext_status = 1      # reset exit to SAVE
                call set_count(rec4_cnt)
                input array q_xerord without defaults from b_xerord.*
                    on key (control-i)
                        let ext_status = 0  # exit is a TAB
                        exit input
                    on key (interrupt)
                        let ext_status = 2  # exit is an INTERRUPT
                        let int_flag = 0
                        exit input
                end input
                if ext_status != 2  # exit is not an INTERRUPT
                then
                    let rec4_cnt = arr_count()
                end if
                if ext_status != 0  # exit is not a TAB
                then
                    exit while
                end if
            end while

            if ext_status != 1      # exit is not a SAVE
            then
                for n = 1 to 25
                    let p_err_line = q_err_line
                    display p_err_line to s_xerorh.err_txt
                    let p_xerord[n].err_text = r_xerord[n].err_text
                    let q_xerord[n].err_text = s_xerord[n].err_text
                    if n < 5
                    then
                        display p_xerord[n].* to a_xerord[n].*
                        attribute(red)
                        display q_xerord[n].* to b_xerord[n].*
                        attribute(red)
                    end if
                end for
            else                    # exit is a SAVE
                for n = 1 to 25
                    let q_err_line = p_err_line
                    let r_xerord[n].err_text = p_xerord[n].err_text
                    let s_xerord[n].err_text = q_xerord[n].err_text
                end for
                let rec1_cnt = rec3_cnt
                let rec2_cnt = rec4_cnt
                delete from stxerord where
                        stxerord.err_module = err_mod
                    and  stxerord.err_program = err_prog
                    and  stxerord.err_number = err_num
                update stxerorh set err_line = p_err_line where
                       err_module = err_mod and err_program = err_prog and
                       err_number = err_num
                for n = 1 to rec1_cnt
                    insert into stxerord values(err_mod,err_prog,err_num,
                                                "a",n,p_xerord[n].err_text)
                end for
                for n = 1 to rec2_cnt
                    insert into stxerord values(err_mod,err_prog,err_num,
                                                "b",n,q_xerord[n].err_text)
                end for
            end if

        command key ("!")
            call ring_bang()

        command key (e,x,q,interrupt) "Quit" " Return to the program"
            if int_flag != 0
            then
                let int_flag = 0
            end if
            exit menu

    end menu

    close window errtxtwin

end function
# err_info()
