########################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../lib.4gm/all.4gs/ic_costv.4gl  1.5  Delta: 9/4/90
########################################################################

globals "ic_glob.4gl"

########################################################################
function cost_put(item_code,warehouse_code,qty,cost,last_cost,p_method)
########################################################################
#
# The purpose of this function is to update or create a stack in the
# cost transaction table.  The stack can be from the head or the tail
# of the link list.
#
    define
        new_qty decimal(10),
        item_code char(15),
        warehouse_code char(3),
        qty decimal(10),
        cost decimal(12),
        last_cost decimal(12),
        p_method char(1),           # Fifo, Lifo, C-cost method in control
        cost_eval record            # cost evaluation record
            hierarchy_no integer,
            item_code char(15),
            warehouse_code char(3),
            quantity decimal(10),
            cost decimal(12)
        end record,
        sql_str char(150),
        prep_status smallint,
        p_use_method char(1)

    let prep_status = true

    if p_method = "C"
      then let p_use_method = post_ic.cost_method
      else let p_use_method = p_method
    end if

    let sql_str = "select hierarchy_no,item_code,warehouse_code,quantity,cost ",
       "from sticstvr where item_code = ? and warehouse_code = ? ",
           "order by hierarchy_no"
    prepare sel2 from sql_str
    if sqlca.sqlcode != 0 then let prep_status = false end if
    declare selec2 scroll cursor for sel2

    let sql_str = "update sticstvr set quantity = ? ",
                  "where hierarchy_no = ?"
    prepare upd7 from sql_str
    if sqlca.sqlcode != 0 then let prep_status = false end if

    let sql_str = "insert into sticstvr values(0,?,?,?,?)"
    prepare ins1 from sql_str
    if sqlca.sqlcode != 0 then let prep_status = false end if

    let sql_str = "insert into sticstvr values(?,?,?,?,?)"
    prepare ins2 from sql_str
    if sqlca.sqlcode != 0 then let prep_status = false end if

    let sql_str = "delete from sticstvr where hierarchy_no = ?"
    prepare del2 from sql_str
    if sqlca.sqlcode != 0 then let prep_status = false end if

    # if there were any problems with the prepares, set "not setup"
    # status and return with bad prepare message.
    if not prep_status
    then
        let sqlca.sqlcode = 0
        let post_ic.status = 11
        let post_ic.description =
            "Cannot Prepare Inventory Control SQL Statements"
    return false
    end if

    open selec2 using item_code, warehouse_code
    if p_method = "L"
    then
        fetch last selec2 into cost_eval.*
    else
        fetch first selec2 into cost_eval.*
    end if
    if sqlca.sqlcode != 0 and sqlca.sqlcode != notfound  # fetch of stack failed
    then
        close selec2
        whenever error call error_handler
        let sqlca.sqlcode = 0
        let post_ic.status = 12
        let post_ic.description = "Cannot Read Cost Transaction Table"
        return false
    end if

    # if the current stack has a negative quantity
    if sqlca.sqlcode != notfound and cost_eval.quantity < 0
    then
         let new_qty = qty + cost_eval.quantity
         if new_qty < 0
         then
             whenever error continue
             execute upd7 using new_qty, cost_eval.hierarchy_no
             whenever error call error_handler
             if sqlca.sqlcode != 0
             then
                 let sqlca.sqlcode = 0
                 let post_ic.status = 14
                 let post_ic.description = "Cannot Update Cost Transaction."
                 return false
             end if
             if post_ic.cost_method != "A"
             then
                 let post_ic.gl_amount = 
                   al_round("a",(qty * cost_eval.cost) - (qty * cost))
             end if
             return true
         else
             if post_ic.cost_method != "A"
             then
                 let post_ic.gl_amount =
                   al_round("a",(qty * cost_eval.cost) - (qty * cost))
             end if
             whenever error continue
             execute del2 using cost_eval.hierarchy_no
             whenever error call error_handler
             if sqlca.sqlcode != 0
             then
                 let sqlca.sqlcode = 0
                 let post_ic.status = 14
                 let post_ic.description = "Cannot Delete Cost Transaction."
                 return false
             end if
             if new_qty = 0 then return true end if
             let qty = new_qty
             let sqlca.sqlcode = notfound
         end if
    end if

    # if stack cost is not the same as the tranaction cost
    if (cost_eval.cost != cost or sqlca.sqlcode = notfound) or new_qty < 0
    then
         whenever error continue
         if p_method = "L" or sqlca.sqlcode = notfound
         then
             execute ins1 using item_code, warehouse_code, qty, cost
         else
             let cost_eval.hierarchy_no = cost_eval.hierarchy_no - 1
             execute ins2 using cost_eval.hierarchy_no,
                                item_code,warehouse_code,qty, cost
         end if
         whenever error call error_handler
         if sqlca.sqlcode != 0
         then
             let sqlca.sqlcode = 0
             let post_ic.status = 13
             let post_ic.description = "Cannot Add Cost Transaction"
             return false
         end if
    else
         let new_qty = cost_eval.quantity + qty
         whenever error continue
         execute upd7 using new_qty, cost_eval.hierarchy_no
         whenever error call error_handler
         if sqlca.sqlcode != 0
         then
             let sqlca.sqlcode = 0
             let post_ic.status = 14
             let post_ic.description = "Cannot Update Cost Transaction."
             return false
         end if
    end if
    return true

end function
# cost_put()

########################################################################
function cost_pop(item_code,warehouse_code,qty,last_cost,p_method)
########################################################################
#
# The purpose of this function is to update or remove a stack in the
# cost transaction table.  The stack can be from the head or the tail
# of the link list.
#
    define
        total_cost decimal(12),
        new_qty decimal(10),
        item_code char(15),
        warehouse_code char(3),
        qty decimal(10),
        last_cost decimal(12),
        p_method char(1),
        cost_eval record            # cost evaluation record
            hierarchy_no integer,
            item_code char(15),
            warehouse_code char(3),
            quantity decimal(10),
            cost decimal(12)
        end record,
        sql_str char(150),
        prep_status smallint,
        del_status smallint,
        p_use_method char(1),
        first_time smallint

    let first_time = true      #first time indicator for while loop
    let prep_status = true

    if p_method = "C"
      then let p_use_method = post_ic.cost_method
      else let p_use_method = p_method
    end if

    let sql_str = "select hierarchy_no,item_code,warehouse_code,quantity,cost ",
       "from sticstvr where item_code = ? and warehouse_code = ? ",
           "order by hierarchy_no"
    prepare sel3 from sql_str
    if sqlca.sqlcode != 0 then let prep_status = false end if
    declare selec3 scroll cursor for sel3

    let sql_str = "update sticstvr set quantity = ? where hierarchy_no = ?"
    prepare upd8 from sql_str
    if sqlca.sqlcode != 0 then let prep_status = false end if

    let sql_str = "delete from sticstvr where hierarchy_no = ?"
    prepare del1 from sql_str
    if sqlca.sqlcode != 0 then let prep_status = false end if

    let sql_str = "insert into sticstvr values(0,?,?,?,?)"
    prepare ins3 from sql_str
    if sqlca.sqlcode != 0 then let prep_status = false end if

    # if there were any problems with the prepares, set "not setup"
    # status and return with bad prepare message.
    if not prep_status
    then
        let sqlca.sqlcode = 0
        let post_ic.status = 11
        let post_ic.description =
            "Cannot Prepare Inventory Control SQL Statements"
        return false
    end if

    open selec3 using item_code, warehouse_code

    if p_use_method = "L"
    then
        fetch last selec3 into cost_eval.*
    else
        fetch first selec3 into cost_eval.*
    end if

    if sqlca.sqlcode != 0 and sqlca.sqlcode != notfound     # fetch failed
    then
        close selec3
        whenever error call error_handler
        let sqlca.sqlcode = 0
        let post_ic.status = 12
        let post_ic.description = "Cannot Read Cost Transaction Table"
        return false
    end if
    if sqlca.sqlcode = notfound
    then
        let qty = qty * -1
        whenever error continue
        execute ins3 using item_code, warehouse_code, qty, last_cost
        whenever error call error_handler
        if sqlca.sqlcode != 0
        then
            let sqlca.sqlcode = 0
            let post_ic.status = 13
            let post_ic.description = "Cannot Add Cost Transaction"
            return false
        end if
        if post_ic.cost_method != "A"
        then
            let post_ic.gl_amount = al_round("a",(qty * last_cost))
        end if
        return true
    end if
    let total_cost = 0
    while qty > 0 or first_time   # remove stacks until transaction qty runs out
        let first_time = false
        let del_status = false
        let new_qty = cost_eval.quantity - qty
        if new_qty <= 0
        then
            let del_status = true
            let new_qty = cost_eval.quantity
            let total_cost = total_cost + 
              al_round("a",(new_qty * cost_eval.cost))
        else
            let total_cost = total_cost + al_round("a", (qty * cost_eval.cost))
        end if
        let qty = qty - cost_eval.quantity
        whenever error continue
        execute upd8 using new_qty, cost_eval.hierarchy_no
        whenever error call error_handler
        if sqlca.sqlcode != 0
        then
            let sqlca.sqlcode = 0
            let post_ic.status = 14
            let post_ic.description = "Cannot Update Cost Transaction."
            return false
        end if
        if del_status
        then
            whenever error continue
            execute del1 using cost_eval.hierarchy_no
            whenever error call error_handler
            if sqlca.sqlcode != 0
            then
                let sqlca.sqlcode = 0
                let post_ic.status = 14
                let post_ic.description = "Cannot Delete Cost Transaction."
                return false
            end if
        end if
        if qty > 0
        then
            if p_use_method = "L"
            then
               fetch previous selec3 into cost_eval.*
            else
               fetch next selec3 into cost_eval.*
            end if
            if sqlca.sqlcode != 0 and sqlca.sqlcode != notfound
            then
                close selec3
                whenever error call error_handler
                let sqlca.sqlcode = 0
                let post_ic.status = 12
                let post_ic.description = "Cannot Read Cost Transaction Table"
                return false
            end if
            if sqlca.sqlcode = notfound
            then
                let qty = qty * -1
                whenever error continue
                execute ins3 using item_code, warehouse_code, qty, last_cost
                whenever error call error_handler
                if sqlca.sqlcode != 0
                then
                    let sqlca.sqlcode = 0
                    let post_ic.status = 13
                    let post_ic.description = "Cannot Add Cost Transaction"
                    return false
                end if
                if post_ic.cost_method != "A"
                then
                  let total_cost = 
                    al_round("a", (-1 * qty * last_cost)) + total_cost
                end if
            end if
        end if
    end while
    if post_ic.cost_method != "A"
    then
        let post_ic.gl_amount = total_cost * -1
    end if
    return true
end function
# cost_pop()
