########################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../lib.4gm/all.4gs/ic_post.4gl  1.15  Delta: 9/4/90
########################################################################

globals "ic_glob.4gl"

define
    p_keep_hist char(1)    # allow posting to on line history.

########################################################################
function ic_post(post_or_check, orig_journal, doc_no,
                 post_no, post_date, doc_date, ref_code, doc_desc,
                 doc_type, ref_no, line_no, item_code, warehouse_code,
                 qty, cost, price, comm_code, item_class)
# returns true/false based on pass/fail of the routine
########################################################################
# This routine takes the I/C transaction data as arguments,
# and either posts to I/C or checks for an ok posting.
# It is designed to be run in "CHECK" mode during the edit list phase,
# and in "POST" mode during the posting phase.
# It modifies elements in the global record post_ic defined in
# ic_glob.4gl in this directory.
# All routines that post into the 4gen general ledger module should
# do so via this routine.
# This line is called once for each I/C transaction
#
# Data elements passed:
#    post_or_check char(5),  "POST" or "CHECK"  tells gl_post to check
#                            the posting (for edit lists), or post it.
#    orig_journal char(2),   original journal (AR, OE, etc...)
#    doc_no integer,         document number (sequential and congruent
#                            for each document within a journal)
#    post_no integer,        posting number (post only)
#    post_date date,         posting date (post only)
#    doc_date date,          document entry date
#    ref_code char(6),       vendor/customer code
#    doc_desc char(30),      document description
#    doc_type char(2),       document type "PU", "SH", "TR", "CT", "AJ"
#                            "PU" = purchase/receive inventory,
#                            "SH" = shipped/sell inventory
#                            "TR" = transfer inventory
#                            "CT" = count adjustment of inventory
#                            "AJ" = adjustment of cost or qty
#                            "PJ" = purchase adjustment
#    ref_no   char(10),      document reference number
#    line_no  smallint,      detail line no
#    item_code char(15),     inventory item_code
#    warehouse_code char(3), warehouse location code
#    qty decimal(10),        transaction qty
#    cost decimal(12),       transaction cost
#    price decimal(12),      transaction price
#    comm_code char(6),      item/warehouse commission code
#    item_class char(3)      item product class
#
#
# status and description variables returned in the post_ic record:
# 0 - Successful
# 1 - fg_error("lib_all","ic_post",1) - I/C isn't installed
#     - return true
# 2 - fg_error("lib_all","ic_post",2) - Cannot Read I/C Control Table.
#     - return false
# 3 - fg_error("lib_all","ic_post",3) - I/C Setup is not Complete
#     - (warning condition only) - continue
# 4 - fg_error("lib_all","ic_post",4) - Duplicate Document Number Exists
#     - (error condition) - warning continue
# 5 - fg_error("lib_all","ic_post",5) - Cannot Insert a New Document
#     - (sql insert command failed) - return false
# 6 - fg_error("lib_all","ic_post",6) - Cannot Find Inventory Item
#     -  return false
# 7 - fg_error("lib_all","ic_post",7) - Cannot Find Warehouse Item
#     -  return false
# 8 - fg_error("lib_all","ic_post",8) - Invalid Document Type
#     -  return false
# 9 - fg_error("lib_all","ic_post",9) - Cannot Update Warehouse Item
#     -  return false
#10 - fg_error("lib_all","ic_post",10) - Cannot Update Warehouse Item History
#     -  return false
#11 - fg_error("lib_all","ic_post",11) - Cannot Prepare Inventory Control SQL Statements
#     -  return false
#
# What this process does:
# Check to see if I/C is installed.  If not, then return true
# If last doc no isn't this doc no (first in a group)
#     if last doc no isnt this doc no - 1, then set warning #4
#     call trx_post - this (possibly) posts a new document into
#         stxtranr (top level)
#         trx_post returns:
#         0 = if "CHECK" or if "POST" and posted ok
#         1 = insert failed (post only)
#         2 = duplicate found
#     if "CHECK", check for duplicate stitranr
#     if "POST", insert into stitranr
# if (first in group)
# if "CHECK", then return
# if "POST" then:
#     insert the row into stiactvd
#     update the qty_on_hand and avg_unit cost in stilocar
#     update the total qty and amount fields in stistatd
#
#
    define
        rec_whse record   # receiving warehouse record
            item_class char(3),
            qty decimal(10),
            avg_cost decimal(12),
            comm_code char(6),
            purch_unit_cost decimal(12),
            last_cost decimal(12)
        end record,
        purch_cost decimal(12),  # working number
        new_qty decimal(10),        # working number
        new_cost decimal(12),  # working number
        new_avg_cost decimal(12),   # working number
        new_amount decimal(12),     # working number
        old_amount decimal(12),     # working number
        post_or_check char(5),
        orig_journal like stxtranr.orig_journal,
        doc_no like stxtranr.doc_no,
        post_no like stxtranr.post_no,
        post_date like stxtranr.post_date,
        doc_date like stxtranr.doc_date,
        ref_code like stxtranr.ref_code,
        doc_desc like stxtranr.doc_desc,
        doc_type char(2),
        ref_no   char(10),
        line_no  smallint,
        item_code char(15),
        warehouse_code char(3),
        qty decimal(10),
        cost decimal(12),
        tr_cost decimal(12),
        price decimal(12),
        comm_code char(6),
        item_class char(3),
        sql_str char(150),
        prep_status smallint,
        doc_status smallint

    let post_ic.status = 0
    # First time... fill post_ic.ic_installed
    if post_ic.ic_installed is null
    then
        whenever error continue
        select tabname into post_ic.description
           from systables where tabname = "sticntrc"
        whenever error call error_handler
        if sqlca.sqlcode != 0
        then
            let sqlca.sqlcode = 0
            let post_ic.ic_installed = "N"
        else
            let post_ic.ic_installed = "Y"

    # If I/C is installed, prepare all the sql statements.
            let prep_status = true
            whenever error continue
            let sql_str =
                "select ic_setup_done, adj_acct_no, curr_period, ",
                   "cost_method, keep_hist ",
                      "from sticntrc"
            prepare sel1 from sql_str
            if sqlca.sqlcode != 0 then let prep_status = false end if
            declare selec1 cursor for sel1
            if sqlca.sqlcode != 0 then let prep_status = false end if

    # Check to see if setup is done and retrieve curr_period for use
    # in later prepares.
            open selec1
            fetch selec1 into
                post_ic.setup_done,
                post_ic.bal_acct_no,
                post_ic.curr_period,
                post_ic.cost_method,
                p_keep_hist
            if sqlca.sqlcode != 0
            then
                close selec1
                whenever error call error_handler
                let sqlca.sqlcode = 0
                let post_ic.setup_done = "N"
                let post_ic.status = 2
                let post_ic.description = "Cannot Read I/C Control Table"
                return false
            end if
            close selec1

            let sql_str = "select count(*) from stitranr ",
                "where orig_journal = ? and doc_no = ?"
            prepare cnt1 from sql_str
            if sqlca.sqlcode != 0 then let prep_status = false end if
            declare count1 cursor for cnt1
            if sqlca.sqlcode != 0 then let prep_status = false end if

            let sql_str = "select ",
               "item_class, inv_acct_no, cog_acct_no, sales_acct_no ",
                  "from stiinvtr where item_code = ?"
            prepare cnt2 from sql_str
            if sqlca.sqlcode != 0 then let prep_status = false end if
            declare count2 cursor for cnt2
            if sqlca.sqlcode != 0 then let prep_status = false end if

            let sql_str = "select ",
                "comm_code, qty_on_hand, avg_unit_cost, ",
                "terms_disc, purch_unit_cost, last_cost ",
                    "from stilocar where item_code = ? and warehouse_code = ?"
            prepare cnt3 from sql_str
            if sqlca.sqlcode != 0 then let prep_status = false end if
            declare count3 cursor for cnt3
            if sqlca.sqlcode != 0 then let prep_status = false end if

            let sql_str = "insert into stitranr values(?,?,?,?)"
            prepare ins1 from sql_str
            if sqlca.sqlcode != 0 then let prep_status = false end if

            let sql_str =
                "insert into stiactvd values(?,?,?,?,?,?,?,?,?,?)"
            prepare ins2 from sql_str
            if sqlca.sqlcode != 0 then let prep_status = false end if

            let sql_str = "update stilocar set ",
                          "last_cost = ?, ",
                          "purchase_date = ?, ",
                          "last_qty = ?, ",
                          "avg_unit_cost = ?, ",
                          "qty_on_hand = ? ",
                          "where item_code = ? and warehouse_code = ?"
            prepare upd1 from sql_str
            if sqlca.sqlcode != 0 then let prep_status = false end if

            let sql_str = "update stistatd set ",
                "prd",post_ic.curr_period using "<<<<<<" clipped, "_cost = ",
                "prd",post_ic.curr_period using "<<<<<<" clipped, "_cost + ?, ",
                "prd",post_ic.curr_period using "<<<<<<" clipped, "_cqty = ",
                "prd",post_ic.curr_period using "<<<<<<" clipped, "_cqty + ?  ",
                "where item_code = ? and warehouse_code = ?"
            prepare upd2 from sql_str
            if sqlca.sqlcode != 0 then let prep_status = false end if

            let sql_str = "update stilocar set ",
                          "sold_date = ?, ",
                          "qty_on_hand = ? ",
                          "where item_code = ? and warehouse_code = ?"
            prepare upd3 from sql_str
            if sqlca.sqlcode != 0 then let prep_status = false end if

            let sql_str = "update stistatd set ",
                "prd",post_ic.curr_period using "<<<<<<" clipped,"_sale = ",
                "prd",post_ic.curr_period using "<<<<<<" clipped,"_sale + ?, ",
                "prd",post_ic.curr_period using "<<<<<<" clipped,"_sqty = ",
                "prd",post_ic.curr_period using "<<<<<<" clipped,"_sqty + ?  ",
                "where item_code = ? and warehouse_code = ?"
            prepare upd4 from sql_str
            if sqlca.sqlcode != 0 then let prep_status = false end if

            let sql_str = "update stilocar set ",
                        "qty_on_hand = ? ",
                        "where item_code = ? and warehouse_code = ?"
            prepare upd5 from sql_str
            if sqlca.sqlcode != 0 then let prep_status = false end if

            let sql_str = "update stilocar set ",
                        "qty_on_hand = ?, ",
                        "avg_unit_cost = ? ",
                        "where item_code = ? and warehouse_code = ?"
            prepare upd6 from sql_str
            if sqlca.sqlcode != 0 then let prep_status = false end if

            whenever error call error_handler

    # if there were any problems with the prepares, set "not setup"
    # status and return with bad prepare message.
            if not prep_status
            then
                let sqlca.sqlcode = 0
                let post_ic.setup_done = "N"
                let post_ic.status = 11
                let post_ic.description =
                    "Cannot Prepare Inventory Control SQL Statements"
                return false
            end if
        end if
    end if

    if post_ic.ic_installed = "N"
    then
        let post_ic.status = 1
        let post_ic.description = "I/C Isn't Installed"
        return true
    end if

    if post_ic.setup_done != "Y"
    then
        let post_ic.status = 3
        let post_ic.description = "I/C Setup Isn't Complete"
        return false
    end if

    # Select defaults from inventory table.
    whenever error continue
    open count2 using item_code
    fetch count2 into rec_whse.item_class, post_ic.inv_acct_no,
                        post_ic.cog_acct_no, post_ic.sales_acct_no
    if sqlca.sqlcode != 0
    then
        close count2
        whenever error call error_handler
        let sqlca.sqlcode = 0
        let post_ic.status = 6
        let post_ic.description = "Cannot Find Inventory Item"
        return false
    end if
    close count2
    whenever error call error_handler

    # Select receiving warehouse defaults from inventory location table.
    if post_or_check = "POST" and (doc_type != "TR" or doc_type != "PU")
    then
        whenever error continue
        open count3 using item_code, warehouse_code
        fetch count3 into rec_whse.comm_code, rec_whse.qty,
                          rec_whse.avg_cost, post_ic.terms_disc,
                          rec_whse.purch_unit_cost, rec_whse.last_cost
        if sqlca.sqlcode != 0
        then
            close count3
            whenever error call error_handler
            let sqlca.sqlcode = 0
            let post_ic.status = 7
            let post_ic.description = "Cannot Find Warehouse Item"
            return false
        end if
        close count3
        whenever error call error_handler
    end if

    if post_or_check = "CHECK"
    then
        let post_ic.status = 0
        return true
    end if

    # Check document number for duplication and out of sequence.
    # Post into stxtranr
    if doc_no = post_ic.last_doc_no
    then else
        # first of a group
        if post_ic.last_doc_no is not null and
           post_ic.last_doc_no != doc_no - 1 and
           post_or_check = "POST"
        then
            let post_ic.status = 4
            let post_ic.description = "Document Number out of Sequence"
        end if
        let post_ic.last_doc_no = doc_no

        call trx_post(post_or_check, orig_journal, doc_no,
                 post_no, post_date, doc_date, ref_code, doc_desc)
            returning doc_status
        if doc_status = 1
        then
            let post_ic.status = 5
            let post_ic.description = "Cannot Insert a New Document.",
                                      " (stxtranr)"
            return false
        end if
        if doc_status = 2
        then
            let post_ic.status = 4
            let post_ic.description = "Duplicate Document Number Exists.",
                                      " (stxtranr)"
            return false
        end if
        if post_or_check = "POST"
        then
            whenever error continue
            open count1 using orig_journal, doc_no
            fetch count1 into post_no
            close count1
            whenever error call error_handler
            if post_no > 0
            then
                let post_ic.status = 4
                let post_ic.description = "Duplicate Document Number Exists.",
                                          " (stitranr)"
                return false
            end if

            # insert the stitranr record
            whenever error continue
            execute ins1 using orig_journal,doc_no,doc_type,ref_no
            whenever error call error_handler
            if sqlca.sqlcode != 0
            then
                let sqlca.sqlcode = 0
                let post_ic.status = 5
                let post_ic.description = "Cannot Insert a New Document.",
                                          " (stitranr)"
                return false
            end if
        end if   # POST
    end if   # first document in batch

    # Post activities
    # insert into the stiactvd row

    if comm_code is null then let comm_code = rec_whse.comm_code end if
    if item_class is null then let item_class = rec_whse.item_class end if
    if doc_type = "TR"
    then
        let tr_cost = cost
        let cost = rec_whse.avg_cost
    end if
    if doc_type = "SH" and cost is null
    then
        let cost = rec_whse.avg_cost
    end if
    if rec_whse.last_cost is null
    then
        let purch_cost = rec_whse.purch_unit_cost
    else
        let purch_cost = rec_whse.last_cost
    end if

    whenever error continue
    execute ins2 using orig_journal,doc_no,line_no,item_code,warehouse_code,
                       qty,cost,price,comm_code,item_class
    whenever error call error_handler
    if sqlca.sqlcode != 0
    then
        let sqlca.sqlcode = 0
        let post_ic.status = 5
        let post_ic.description = "Cannot Insert a New Document.",
                                  " (stiactvd)"
    return false
    end if

    # update warehouse inventory item

    case
        when doc_type = "PU"

            if rec_whse.qty is null then let rec_whse.qty = 0 end if
            if rec_whse.avg_cost is null then let rec_whse.avg_cost = 0 end if
            let new_qty = rec_whse.qty + qty

            # if transaction qty + qty on hand is equal to 0 then
            # let avg unit cost = 0 and calculate the adjustment amount to G/L
            # if qty on hand is a negative number then
            # let avg unit cost = transaction unit cost and calculate
            # the adjustment amount to G/L
            # otherwise calculate the new avg unit cost

            if new_qty != 0
            then
                if new_qty < 0
                then
                    let new_avg_cost = cost
                    let post_ic.gl_amount = al_round("a",
                        (rec_whse.qty + qty) * new_avg_cost)
                    let old_amount = al_round("a",
                        (rec_whse.qty * rec_whse.avg_cost) + (qty * cost))
                    let post_ic.gl_amount = post_ic.gl_amount - old_amount
                else
                    let new_avg_cost = al_round("b",
                        ((rec_whse.qty * rec_whse.avg_cost) +
                        (qty * cost)) / (rec_whse.qty + qty))
                    let post_ic.gl_amount = 0
                end if
            else
                let new_avg_cost = 0
                let post_ic.gl_amount = al_round("a",
                    (qty * rec_whse.avg_cost) - (rec_whse.qty * cost))
            end if

            # update warehouse location information
            whenever error continue
            execute upd1 using cost, doc_date, qty, new_avg_cost,
                            new_qty, item_code, warehouse_code
            whenever error call error_handler
            if sqlca.sqlcode != 0
            then
                let sqlca.sqlcode = 0
                let post_ic.status = 9
                let post_ic.description = "Cannot Update Warehouse Item"
                return false
            end if

            # update warehouse history information
            if p_keep_hist = "Y"
            then
              let new_amount = al_round("a",qty * cost)
              whenever error continue
              execute upd2 using new_amount, qty, item_code, warehouse_code
              whenever error call error_handler
              if sqlca.sqlcode != 0
              then
                  let sqlca.sqlcode = 0
                  let post_ic.status = 10
                let post_ic.description = "Cannot Update Warehouse Item History"
                  return false
              end if
            end if

            # FIFO/LIFO transaction
            call cost_put(item_code,warehouse_code, qty,cost,
                purch_cost,"L") returning prep_status
            if not prep_status then return false end if

        when doc_type = "SH"

            if rec_whse.qty is null then let rec_whse.qty = 0 end if
            let new_qty = rec_whse.qty - qty

            # qty on hand - transaction qty
            # calculate transaction amount to G/L

            # update warehouse location information
            whenever error continue
            execute upd3 using doc_date, new_qty, item_code, warehouse_code
            whenever error call error_handler
            if sqlca.sqlcode != 0
            then
                let sqlca.sqlcode = 0
                let post_ic.status = 9
                let post_ic.description = "Cannot Update Warehouse Item"
                return false
            end if

            # update warehouse history information
            if p_keep_hist = "Y"
            then
              let new_amount = al_round("a",qty * price)
              whenever error continue
              execute upd4 using new_amount, qty, item_code, warehouse_code
              whenever error call error_handler
              if sqlca.sqlcode != 0
              then
                let sqlca.sqlcode = 0
                let post_ic.status = 10
                let post_ic.description = "Cannot Update Warehouse Item History"
                return false
              end if
            end if

            let post_ic.gl_amount = al_round("a",
                qty * rec_whse.avg_cost * -1)

            # FIFO/LIFO transaction
            call cost_pop(item_code,warehouse_code,
                qty,purch_cost,"C") returning prep_status
            if not prep_status then return false end if

        when doc_type = "TR"
            # if cost is null then assume the transaction is for moving qty
            # from the warehouse qty on hand - transaction qty
            # otherwise the qty is moving to the warehouse

            if tr_cost is null
            then
                if rec_whse.qty is null then let rec_whse.qty = 0 end if
                let new_qty = rec_whse.qty - qty
                # update warehouse location information
                whenever error continue
                execute upd5 using new_qty, item_code, warehouse_code
                whenever error call error_handler
                if sqlca.sqlcode != 0
                then
                    let sqlca.sqlcode = 0
                    let post_ic.status = 9
                    let post_ic.description = "Cannot Update Warehouse Item"
                    return false
                end if
                let post_ic.gl_amount = al_round("a",
                    qty * rec_whse.avg_cost * -1)

                # FIFO/LIFO transaction
                call cost_pop(item_code,warehouse_code,
                    qty,purch_cost,"C") returning prep_status
                if not prep_status then return false end if
            else
                # the tranfer cost is determined by the 
                #   (previous gl_amount)/(tranfer qty)

                if rec_whse.qty is null then let rec_whse.qty = 0 end if
                if rec_whse.avg_cost is null 
                then 
                  let new_avg_cost = al_round("a",
                                       ((post_ic.gl_amount)/qty))
                else
                  let new_avg_cost = al_round("a",
                   ((rec_whse.qty * rec_whse.avg_cost) + 
                    (post_ic.gl_amount))/(qty+rec_whse.qty))
                end if
                let new_qty = rec_whse.qty + qty

                # update warehouse location information
                whenever error continue
                execute upd6 
                  using new_qty, new_avg_cost, item_code, warehouse_code
                whenever error call error_handler
                if sqlca.sqlcode != 0
                then
                    let sqlca.sqlcode = 0
                    let post_ic.status = 9
                    let post_ic.description = "Cannot Update Warehouse Item"
                    return false
                end if

                let new_amount = al_round("a",((post_ic.gl_amount/qty)))

                # FIFO/LIFO transaction
                call cost_put(item_code,warehouse_code,
                    qty,new_amount,purch_cost,"C") returning prep_status
                if not prep_status then return false end if
            end if

        when doc_type = "AJ"
            case
                # adjustment of qty, qty on hand + transaction qty
                when cost is null
                    if rec_whse.qty is null then let rec_whse.qty = 0 end if
                    let new_qty = rec_whse.qty + qty
                    # update warehouse location information
                    whenever error continue
                    execute upd5 using new_qty, item_code, warehouse_code
                    whenever error call error_handler
                    if sqlca.sqlcode != 0
                    then
                        let sqlca.sqlcode = 0
                        let post_ic.status = 9
                        let post_ic.description = "Cannot Update Warehouse Item"
                        return false
                    end if
                    let post_ic.gl_amount = al_round("a",
                        qty * rec_whse.avg_cost)

                    let new_qty = qty * -1

                    # FIFO/LIFO transaction
                    call cost_pop(item_code,warehouse_code,new_qty,
                        purch_cost,"C") returning prep_status
                    if not prep_status then return false end if

                # adjustment of number of transaction qty at a given cost
                otherwise

                    let new_avg_cost = al_round( "a", (((rec_whse.qty - qty) *
                        rec_whse.avg_cost) + (qty * cost)) / (rec_whse.qty) )
                    # update warehouse location information
                    whenever error continue
                    execute upd6 using rec_whse.qty, new_avg_cost, 
                                       item_code, warehouse_code
                    whenever error call error_handler
                    if sqlca.sqlcode != 0
                    then
                        let sqlca.sqlcode = 0
                        let post_ic.status = 9
                        let post_ic.description = "Cannot Update Warehouse Item"
                        return false
                    end if
                    let post_ic.gl_amount = al_round("a",
                       ((rec_whse.qty * rec_whse.avg_cost) -
                        (rec_whse.qty * new_avg_cost)) * -1) 

                    # FIFO/LIFO transaction
                    call cost_pop(item_code,warehouse_code,
                        qty,purch_cost,"C") returning prep_status
                    if not prep_status then return false end if
                    if post_ic.cost_method != "A"
                    then
                        let post_ic.gl_amount = al_round("a",
                            qty * cost) + post_ic.gl_amount
                    end if
                    call cost_put(item_code,warehouse_code,
                                  qty,cost,purch_cost,"C")
                        returning prep_status
                    if not prep_status then return false end if

                end case

        when doc_type = "PJ"
            # purchase adjustments, 
            # the qty is positive if under paid of inventory
            # the qty is negative if over paid of inventory
            # the cost if the amount over/under paid
            # this section will remove the qty from inventory and 
            #   add the same amount back with the new cost.
            # it will not set the gl.amount, pu posting will handle this
            if qty < 0 
            then
              let new_cost = rec_whse.avg_cost - cost
              let new_qty = qty * -1
            else
              let new_cost = rec_whse.avg_cost + cost
              let new_qty = qty
            end if
          
            let new_avg_cost = al_round( "a", (((rec_whse.qty-new_qty) *
                rec_whse.avg_cost) + (new_qty * new_cost)) / (rec_whse.qty) )
            # update warehouse location information
            whenever error continue
            execute upd6 using new_avg_cost, item_code, warehouse_code
            whenever error call error_handler
            if sqlca.sqlcode != 0
            then
                let sqlca.sqlcode = 0
                let post_ic.status = 9
                let post_ic.description = "Cannot Update Warehouse Item"
                return false
            end if
            # FIFO/LIFO transaction
            call cost_pop(item_code,warehouse_code,
                new_qty,purch_cost,"C") returning prep_status
            if not prep_status then return false end if
            if qty < 0 
            then
             let new_cost = al_round("a",((post_ic.gl_amount / new_qty) - cost))
            else
             let new_cost = al_round("a",((post_ic.gl_amount / new_qty) + cost))
            end if
            call cost_put(item_code,warehouse_code,
                          new_qty,new_cost,purch_cost,"C")
                returning prep_status
            if not prep_status then return false end if

        when doc_type = "CT"

            # update warehouse location information
            whenever error continue
            execute upd5 using qty, item_code, warehouse_code
            whenever error call error_handler
            if sqlca.sqlcode != 0
            then
                let sqlca.sqlcode = 0
                let post_ic.status = 9
                let post_ic.description = "Cannot Update Warehouse Item"
                return false
            end if

            if rec_whse.qty is null then let rec_whse.qty = 0 end if
            if rec_whse.avg_cost is null then let rec_whse.avg_cost = 0 end if
            let post_ic.gl_amount = al_round("a",
                (qty * rec_whse.avg_cost) -
                (rec_whse.qty * rec_whse.avg_cost))

            let new_qty = rec_whse.qty - qty

            # FIFO/LIFO transaction
            call cost_pop(item_code,warehouse_code,
                          new_qty,purch_cost,"L") returning prep_status
            if not prep_status then return false end if

        otherwise
            let post_ic.status = 8
            let post_ic.description = "Invalid Document Type"
            return false

    end case

    return true
end function
# ic_post()

