########################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../lib.4gm/all.4gs/pu_post.4gl  1.2  Delta: 9/4/90
########################################################################


globals "pu_glob.4gl"

########################################################################

function pu_post(post_or_check, orig_journal, doc_no, post_no,
    post_date, doc_date, ref_code, doc_desc, inv_no, doc_type,
    trd_ds_code, buyer_code, act_type,
    line_no, cm_dm_reason, reason_type, item_code, warehouse_code,
    quantity, amount, payment, pay_method, rcpt_date, exp_rcpt_date,
    requisition_no, authorization_code, inspection_code)

# returns true/false based on pass/fail of the routine
########################################################################
#  This routine takes the Purchase Order transaction data as arguments,
#  and either posts to Purchase Order or checks for an ok posting.
#  It is designed to be run in "CHECK" mode during the edit list phase,
#  and in "POST" mode during the posting phase.  It modifies elements in
#  the global record post_pu defined in pu_glob.4gl in this directory.
#  All routines that post into the 4gen Purchase Order module should do
#  so via this routine.
#
#  There are three main types of transactions this processes: invoices,
#  credit memos and debit memos.  There is very little difference
#  in the way they are processed, other than that the amounts for
#  credit memos are negative, and credit and debit memos have a reason
#  code.  There is also a transaction stored when a purchase order is
#  prepaid, or when a prepaid purchase order is cancelled.  These don't
#  affect anything but accounts payable and general ledger, so the
#  transactions are stored here for information purposes only.  they
#  have only one activity record for the prepay amount.
#
#  This function is called for each non-comment line and for each non-
#  zero order amount (freight, miscellaneous, city, county and state
#  taxes, and trade discount).  It creates a row in stxtranr, if one
#  does not exist; creates a row in stutranr the first time through;
#  and creates a row in stuactvd for each time it is called.
#
#  This routine uses prepares/declares/and executes that build the
#    sql query from a string because it needs to be able to compile
#    without any Purchase Order files installed.
#
#  Data elements passed:
#    post_or_check char(5),  "POST" or "CHECK"  tells pu_post to check
#                            the posting (for edit lists), or post it.
#    orig_journal char(2),   original journal (AP, PU, etc...)
#    doc_no integer,         invoice document number (sequential and
#                            congruent for each document within a
#                            journal)
#    post_no integer,        posting number (post only)
#    post_date date,         posting date (post only)
#    doc_date date,          document entry/billing date
#    ref_code char(6),       vendor code
#    doc_desc char(30),      document description
#    inv_no char(10),        invoice/credit memo/debit memo number
#    doc_type char(2),       Document Types:
#                            IN = Invoice, CM = Credit Memo,
#                            DM = Debit Memo, PP = Prepay,
#                            CP = Canceled Prepay
#    trd_ds_code char(6),    trade discount code.  this is often used
#                            as a vendor type or vendor class code
#    buyer_code char(6),     buyer code.  for purchasing reporting.
#    act_type char(1),       Activity types:
#                            I = item: one for each order item line
#                            F = freight; one for the whole order
#                            T = tax; one for each tax (up to three)
#                            M = miscellaneous; one for the whole order
#                            D = trade discount; one for the whole order
#                                this amount has the sign reversed.
#                            Z = total amount; one for the whole order
#    line_no smallint,       line number for item type.  1, 2, or 3 for
#                            state, county, or city tax type.  0 for
#                            everything else.
#    cm_dm_reason char(3),   reason code for debit or credit memo.  only
#                            has meaning for type "I", doc_type "CM/DM"
#    reason_type smallint,   reason type of cm_dm_reason (1-4).  this is
#                            set to 0 when unneeded.
#    item_code char(20),     item code for type "I", tax code for type
#                            "T", trade discount code for type "D".
#                            credit card number for type "Z".
#                            has no meaning for other types.
#    warehouse_code char(3), warehouse code for item type.  also filled
#                            in for other types.  thus analysis of
#                            freight by warehouse could be done, but it
#                            won't be reliable when different warehouses
#                            are used for different items.
#    quantity decimal(12),   item quantity for item type.
#    amount decimal(12),     signed amount.  this function reverses the
#                            sign for credit memos.  do not specifically
#                            send negative amounts for credit memos.
#    payment char(1),        payment type.  "V" for credit card,
#                                           "C" for cash,
#                                           "A" for a/p account
#    pay_method char(6),     payment method
#    exp_rcpt_date date,     date the item was expected (for type "I")
#    rcpt_date date          date the item was received (for type "I")
#    requisition_no char(10), from the original requisition
#    authorization_code char(8), from the original requisition
#    inspection_code char(8) the userid of the inspector (for type "I")
#
# status and description variables returned in the post_pu record:
# 0 - Successful
# 1 - fg_error("lib_all","pu_post",1) - Purchasing isn't installed
#     - return true
# 2 - fg_error("lib_all","pu_post",2) - Document Number out of Sequence
#     - (warning condition only) - continue
# 3 - fg_error("lib_all","pu_post",3) - Duplicate Document Number Exists
#     - (error condition) - return false
# 4 - fg_error("lib_all","pu_post",4) - Cannot Insert a New Document
#     - (sql insert command failed) - return false
#
# What this process does:
# Check to see if Purchasing is installed.  If not, just return true...
# If last doc no isn't this doc no (first in a group)
#     if last doc no isnt this doc no - 1, then set warning #2
#     call trx_post - this (possibly) posts a new document into
#         stxtranr (top level)
#         trx_post returns:
#         0 = if "CHECK" or if "POST" and posted ok
#         1 = insert failed (post only)
#         2 = dup found
#     if "CHECK", check for duplicate stutranr
#     if "POST", insert into stutranr
# if (first in group)
# if "CHECK", then return
# if "POST" then:
#     if credit memo type
#     then reverse the sign of the amount
#     end if
#     add record to stuactvd
# if
#
    define
        post_or_check char(5),
        orig_journal like stxtranr.orig_journal,
        doc_no like stxtranr.doc_no,
        post_no like stxtranr.post_no,
        post_date like stxtranr.post_date,
        doc_date like stxtranr.doc_date,
        ref_code like stxtranr.ref_code,
        doc_desc like stxtranr.doc_desc,
        inv_no char(10),
        doc_type char(2),
        trd_ds_code like stxinfor.src_key,
        buyer_code like stxinfor.src_key,
        act_type char(1),
        line_no smallint,
        cm_dm_reason like stxinfor.src_key,
        reason_type smallint,
        item_code char(20),
        warehouse_code like stxinfor.src_key,
        quantity decimal(10),
        amount like stgactvd.amount,
        pay_method like stxinfor.src_key,
        payment char(1),
        exp_rcpt_date date,
        rcpt_date date,
        requisition_no char(10),
        authorization_code char(8),
        inspection_code char(8),
        sql_str char(500),
        stat_flag smallint,
        doc_status smallint

    let post_pu.status = 0
    # First time... fill post_pu.pu_installed and post_pu.pu_balanced
    if post_pu.pu_installed is null
    then
        let post_pu.pu_installed = "Y"
        whenever error continue
        select tabname into post_pu.description
           from systables where tabname = "stucntrc"
        whenever error call error_handler
        if sqlca.sqlcode != 0
        then
            let sqlca.sqlcode = 0
            let post_pu.pu_installed = "N"
        else

# set up all the query statements for later file accesses
            let sql_str = "select count(*) from stutranr",
                " where orig_journal = ? and doc_no = ?"
            whenever error continue
            prepare cnt1 from sql_str
            if sqlca.sqlcode != 0
            then
                let sqlca.sqlcode = 0
                let post_pu.pu_installed = "N"
            end if
            declare count1 cursor for cnt1
            if sqlca.sqlcode != 0
            then
                let sqlca.sqlcode = 0
                let post_pu.pu_installed = "N"
            end if
            let sql_str =
                "insert into stutranr ",
                   "(stutranr.orig_journal, ",
                    "stutranr.doc_no, ",
                    "stutranr.inv_no, ",
                    "stutranr.doc_type, ",
                    "stutranr.trd_ds_code, ",
                    "stutranr.buyer_code) ",
                   "values (?, ?, ?, ?, ?, ?)"
            prepare ins1 from sql_str
            if sqlca.sqlcode != 0
            then
                let sqlca.sqlcode = 0
                let post_pu.pu_installed = "N"
            end if
            let sql_str =
                "insert into stuactvd ",
                  "(stuactvd.orig_journal, ",
                  "stuactvd.doc_no, ",
                  "stuactvd.act_type, ",
                  "stuactvd.line_no, ",
                  "stuactvd.cm_dm_reason, ",
                  "stuactvd.reason_type, ",
                  "stuactvd.item_code, ",
                  "stuactvd.warehouse_code, ",
                  "stuactvd.quantity, ",
                  "stuactvd.amount, ",
                  "stuactvd.pay_method, ",
                  "stuactvd.payment, ",
                  "stuactvd.exp_rcpt_date, ",
                  "stuactvd.rcpt_date, ",
                  "stuactvd.requisition_no, ",
                  "stuactvd.authorization_code, ",
                  "stuactvd.inspection_code) ",
                "values ",
                  "(?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)"
            prepare ins2 from sql_str
            if sqlca.sqlcode != 0
            then
                let sqlca.sqlcode = 0
                let post_pu.pu_installed = "N"
            end if
            whenever error call error_handler
        end if
    end if

    if post_pu.pu_installed = "N"
    then
        let post_pu.status = 1
        let post_pu.description = "Purchasing Isn't Installed"
        return true
    end if

    if doc_no = post_pu.last_doc_no
    then else
        # first of a group
        if post_pu.last_doc_no is not null
           and post_pu.last_doc_no != doc_no - 1
           and post_or_check = "POST"
        then
            let post_pu.status = 2
            let post_pu.description = "Document Number out of Sequence"
        end if
        let post_pu.last_doc_no = doc_no
        call trx_post(post_or_check, orig_journal, doc_no,
                 post_no, post_date, doc_date, ref_code, doc_desc)
            returning doc_status
        if doc_status = 1
        then
            let post_pu.status = 4
            let post_pu.description = "Cannot Insert a New Document.",
                                      " (stxtranr)"
            return false
        end if
        if doc_status = 2
        then
            let post_pu.status = 3
            let post_pu.description =
                "Duplicate Document Number Exists. (stxtranr)"
            return false
        end if

        if post_or_check = "POST"
        then
            whenever error continue
            open count1 using orig_journal, doc_no
            fetch count1 into post_no
            close count1
            whenever error call error_handler
            if post_no > 0
            then
                let post_pu.status = 3
                let post_pu.description =
                    "Duplicate Document Number Exists. (stutranr)"
                return false
            end if

            # ok to insert...
            whenever error continue
            execute ins1 using
                orig_journal,
                doc_no,
                inv_no,
                doc_type,
                trd_ds_code,
                buyer_code
            whenever error call error_handler
            if sqlca.sqlcode != 0
            then
                let sqlca.sqlcode = 0
                let post_pu.status = 4
                let post_pu.description =
                    "Cannot Insert a New Document. (stutranr)"
                return false
            end if
        end if   # POST
    end if   # first document in batch

# if just checking, we're done
    if post_or_check = "CHECK"
    then
        let post_pu.status = 0
        return true
    end if

# POST type...
# set any necessary defaults, reverse any necessary signs, insert the
# stractvd row.
    if line_no is null then let line_no = 0 end if
    if quantity is null then let quantity = 1 end if
    if reason_type is null then let reason_type = 0 end if

    if doc_type = "CM"
    then
        let quantity = -1 * quantity
        let amount = -1 * amount
    end if

    if act_type = "D"
    then
        let quantity = -1 * quantity
        let amount = -1 * amount
    end if

    whenever error continue
    execute ins2 using
        orig_journal,
        doc_no,
        act_type,
        line_no,
        cm_dm_reason,
        reason_type,
        item_code,
        warehouse_code,
        quantity,
        amount,
        pay_method,
        payment,
        exp_rcpt_date,
        rcpt_date,
        requisition_no,
        authorization_code,
        inspection_code
    whenever error call error_handler
    if sqlca.sqlcode != 0
    then
        let sqlca.sqlcode = 0
        let post_pu.status = 4
        let post_pu.description =
            "Cannot Insert a New Document. (stuactvd)"
        return false
    end if

    return true
end function
# pu_post()

