########################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../lib.4gm/all.4gs/py_post.4gl  1.7  Delta: 9/4/90
########################################################################

globals "py_glob.4gl"

########################################################################
function py_post(post_or_check, orig_journal, doc_no,
                 post_no, post_date, doc_date, pay_date, ref_code,
                 doc_desc, check_no, act_code, act_type, amount,
                 acct_no, dept_code, eop_date, hours, rate)
# returns true/false based on pass/fail of the routine
########################################################################
#  This routine takes the Payroll transaction data as arguments,
#  and either posts to Payroll or checks for an ok posting.
#  It is designed to be run in "CHECK" mode during the edit list phase,
#  and in "POST" mode during the posting phase.  It modifies elements
#  in the global record post_py defined in py_glob.4gl in this
#  directory.  All routines that post into the 4gen Payroll
#  module should do so via this routine.
#
#  Payroll entries call this routine once per invoice.
#
#  This routine uses prepares/declares/and executes that build the
#    sql query from a string because it needs to be able to compile
#    without any Payroll files installed.
#
#  Data elements passed:
#    post_or_check char(5),  "POST" or "CHECK"  tells py_post to check
#                            the posting (for edit lists), or post it.
#    orig_journal char(2),   original journal (PY, AR, OE, etc...)
#    doc_no integer,         document number (sequential and congruent
#                            for each document within a journal)
#    post_no integer,        posting number (post only)
#    post_date date,         posting date (post only)
#    doc_date date,          document entry date
#    pay_date char(6),       payroll date
#    ref_code char(6),       employee code
#    doc_desc char(30),      document description
#    check_no char(10),      check# (freeform - not used as a key)
#    act_code char(6),       activity code (income, deduction, obligation code)
#    act_type char(1),       activity types:
#                            A = check
#                            B = income
#                            C = deduction
#                            D = employer obligation
#                            E = employer obligation (liability)
#    amount like stgactvd.amount,   signed amount (+increases, -decreases balances)
#    acct_no integer,        posting account number
#    dept_code               posting department
#    eop_date                pay period ending date
#    hours                   hours worked for income (used by Income and Check
#                            activity types - Check stores the total)
#    rate                    rate used in amount calculation - needed primarily
#                            for Income types but used when applicabale by other
#
# status and description variables returned in the post_py record:
# 0 - Successful
# 1 - fg_error("lib_all","py_post",1) - Payroll isn't installed
#     - return true
# 2 - fg_error("lib_all","py_post",2) - Cannot Read Payroll Control Table.
#     - return false
# 4 - fg_error("lib_all","py_post",4) - Document Number out of Sequence
#     - (warning condition only) - continue
# 5 - fg_error("lib_all","py_post",5) - Duplicate Document Number Exists
#     - (error condition) - return false
# 6 - fg_error("lib_all","py_post",6) - Cannot Insert a New Document
#     - (sql insert command failed) - return false
# 7 - fg_error("lib_all","py_post",7) - Cannot Update Deduction Accumulation.
#     - return false
# 8 - fg_error("lib_all","py_post",8) - Cannot Update Obligation Accumulation.
#     - return false
#
# What this process does:
#
# check to see if Payroll is installed.  If not, just return true...
# if last doc no isn't this doc no (first in a group)
#     if last doc no isn't this doc no - 1, then set warning #4
#     call trx_post - this (possibly) posts a new document into
#         stxtranr (top level)
#         trx_post returns:
#         0 = if "CHECK" or if "POST" and posted ok
#         1 = insert failed (post only)
#         2 = dup found
#     if "CHECK", check for duplicate stytranr
#     if "POST", insert into stytranr
# if (first in group)
# if "POST" insert into styactvd
# return
#
# This process is designed to be run in conjunction with the py_last()
# which checks to make sure the entire transaction balances.
#
    define
        post_or_check char(5),
        orig_journal like stxtranr.orig_journal,
        doc_no like stxtranr.doc_no,
        post_no like stxtranr.post_no,
        post_date like stxtranr.post_date,
        doc_date like stxtranr.doc_date,
        pay_date like stxtranr.doc_date,
        ref_code like stxtranr.ref_code,
        doc_desc like stxtranr.doc_desc,
        check_no char(10),
        act_code char(6),
        act_type char(1),
        amount like stgactvd.amount,
        acct_no like stgactvd.acct_no,
        dept_code like stgactvd.department,
        eop_date date,
        hours decimal(12,2),
        rate decimal(18,8),
        stat_flag smallint,         # update status/wait flag
        sql_str char(200),          # query construction string
        doc_status smallint

    # initialize status
    let post_py.status = 0

    # first time... fill post_py.py_installed and post_py.py_ein_number
    if post_py.py_installed is null
    then
        whenever error continue
        select tabname into post_py.description
           from systables where tabname = "stycntrc"
        whenever error call error_handler
        if sqlca.sqlcode != 0
        then
            let sqlca.sqlcode = 0
            let post_py.py_installed = "N"
        else
            let post_py.py_installed = "Y"
        end if
        if post_py.py_installed = "Y"
        then
            let sql_str = "select ein_number from stycntrc"
            whenever error continue
            prepare sel1 from sql_str
            declare selec1 cursor for sel1
            foreach selec1 into post_py.py_ein_number
            end foreach
            whenever error call error_handler
            if sqlca.sqlcode != 0
            then
                let sqlca.sqlcode = 0
                let post_py.status = 2
                let post_py.description = "Cannot read payroll ",
                    "control table"
                return false
            end if
        end if
    end if

    if post_py.py_installed = "N"
    then
        let post_py.status = 1
        let post_py.description = "Payroll isn't installed."
        return true
    end if

    # check for first of group
    if doc_no = post_py.last_doc_no
    then else
        # initialize variables
        let post_py.py_c_accum = 0
        let post_py.py_i_accum = 0
        let post_py.py_d_accum = 0
        let post_py.py_ox_accum = 0
        let post_py.py_ol_accum = 0

        # first of a group
        if post_py.last_doc_no != doc_no - 1
        then
            let post_py.status = 4
            let post_py.description = "Document number out of sequence."
        end if
        let post_py.last_doc_no = doc_no

        # post to stxtranr
        call trx_post(post_or_check, orig_journal, doc_no,
                 post_no, post_date, doc_date, ref_code, doc_desc)
            returning doc_status
        if doc_status = 1
        then
            let post_py.status = 6
            let post_py.description = "Cannot insert a new document."
            return false
        end if
        if doc_status = 2
        then
            let post_py.status = 5
            let post_py.description = "Duplicate document number exists",
                " in stxtranr."
            return false
        end if

        # post to stytranr
        if post_or_check = "POST"
        then
            # check for existing entry
            let sql_str = "select count(*) from stytranr",
                " where orig_journal = ? and doc_no = ?"
            whenever error continue
            prepare cnt1 from sql_str
            declare count1 cursor for cnt1
            open count1 using orig_journal, doc_no
            fetch next count1 into post_no
            close count1
            whenever error call error_handler
            if post_no > 0
            then
                let post_py.status = 5
                let post_py.description = "Duplicate document number ",
                    "exists in stytranr."
                return false
            end if

            # ok to insert...
            whenever error continue
            let sql_str = "insert into stytranr values (?, ?, ?, ?, ?)"
            whenever error continue
            prepare ins1 from sql_str
            execute ins1 using orig_journal, doc_no, check_no, pay_date,
                eop_date
            whenever error call error_handler
            if sqlca.sqlcode != 0
            then
                let sqlca.sqlcode = 0
                let post_py.status = 6
                let post_py.description = "Cannot insert a new document."
                return false
            end if
        end if   # POST
    end if   # first document in batch

    # accumulate check amounts
    case
    when act_type = "A"
        let post_py.py_c_accum = post_py.py_c_accum + amount
    when act_type = "B"
        let post_py.py_i_accum = post_py.py_i_accum + amount
    when act_type = "C"
        let post_py.py_d_accum = post_py.py_d_accum + amount
    when act_type = "D"
        let post_py.py_ox_accum = post_py.py_ox_accum + amount
    when act_type = "E"
        let post_py.py_ol_accum = post_py.py_ol_accum + amount
    end case

    # return true on check
    if post_or_check = "CHECK"
    then
        let post_py.status = 0
        return true
    end if

    # POST type...
    # insert the styactvd row
    let sql_str = "insert into styactvd values (?, ?, ?, ?, ?, ?, ?, ?, ?)"
    whenever error continue
    prepare ins2 from sql_str
    execute ins2 using
        orig_journal, doc_no, act_code, act_type, amount, hours, rate,
        acct_no, dept_code
    whenever error call error_handler
    if sqlca.sqlcode != 0
    then
        let sqlca.sqlcode = 0
        let post_py.status = 6
        let post_py.description = "Cannot insert a new document."
        return false
    end if

    # update accumulation for the income
    if act_type = "B"
    then
        # build the update command
        let sql_str = "update styempid ",
            "set inc_accrual = inc_accrual + ? ",
            "where styempid.empl_code = ? and styempid.inc_code = ?"
        let stat_flag = true
        whenever error continue
        prepare upd0 from sql_str
        execute upd0 using amount, ref_code, act_code
        whenever error call error_handler

        # wait for table/row to free
        if sqlca.sqlcode != 0
        then
            let sqlca.sqlcode = 0
            let stat_flag = false
            open window wt1_win at 10,10 with 1 rows, 60 columns
                attribute(white,border)
            display " Waiting for income: '", act_code clipped,
                "' to become available..." at 1,1
            sleep 5
        end if
        let int_flag = 0
        while stat_flag = false
            whenever error continue
            execute upd0 using amount, ref_code, act_code
            whenever error call error_handler
            if sqlca.sqlcode = 0
            then
                exit while
            else
                let sqlca.sqlcode = 0
                if int_flag != 0
                then
                    let post_py.status = 7
                    let post_py.description = "Cannot update ",
                        "income accumulation."
                    return false
                end if
                sleep 5
            end if
        end while
        if stat_flag = false
        then
            close window wt1_win
        end if
    end if # income

    # update accumulation and last taken if a deduction
    if act_type = "C"
    then
        # build the update command
        let sql_str = "update styempdd ",
            "set (ded_accrual, ded_date) = (ded_accrual + ?, ?) ",
            "where styempdd.empl_code = ? and styempdd.ded_code = ?"
        let stat_flag = true
        whenever error continue
        prepare upd1 from sql_str
        execute upd1 using amount, eop_date, ref_code, act_code
        whenever error call error_handler

        # wait for table/row to free
        if sqlca.sqlcode != 0
        then
            let stat_flag = false
            let sqlca.sqlcode = 0
            open window wt1_win at 10,10 with 1 rows, 60 columns
                attribute(white,border)
            display " Waiting for deduction: '", act_code clipped,
                "' to become available..." at 1,1
            sleep 5
        end if
        let int_flag = 0
        while stat_flag = false
            whenever error continue
            execute upd1 using amount, eop_date, ref_code, act_code
            whenever error call error_handler
            if sqlca.sqlcode = 0
            then
                exit while
            else
                let sqlca.sqlcode = 0
                if int_flag != 0
                then
                    let post_py.status = 7
                    let post_py.description = "Cannot update ",
                        "deduction accumulation."
                    return false
                end if
                sleep 5
            end if
        end while
        if stat_flag = false
        then
            close window wt1_win
        end if
    end if # deduction

    # update accumulation if an obligation
    if act_type = "D"
    then
        # build the update command
        let sql_str = "update styempod ",
            "set obl_accrual = obl_accrual + ? ",
            "where styempod.empl_code = ? and styempod.obl_code = ?"
        let stat_flag = true
        whenever error continue
        prepare upd2 from sql_str
        execute upd2 using amount, ref_code, act_code
        whenever error call error_handler

        # wait for table/row to free
        if sqlca.sqlcode != 0
        then
            let stat_flag = false
            let sqlca.sqlcode = 0
            open window wt1_win at 10,4 with 1 rows, 60 columns
                attribute(white,border)
            display " Waiting for obligation: '", act_code clipped,
                "' to become available..." at 1,1
            sleep 5
        end if
        let int_flag = 0
        while stat_flag = false
            whenever error continue
            execute upd2 using amount, ref_code, act_code
            whenever error call error_handler
            if sqlca.sqlcode = 0
            then
                exit while
            else
                let sqlca.sqlcode = 0
                if int_flag != 0
                then
                    let post_py.status = 8
                    let post_py.description = "Cannot update ",
                        "obligation accumulation."
                    return false
                end if
                sleep 5
            end if
        end while
        if stat_flag = false
        then
            close window wt1_win
        end if
    end if # obligation

    return true
end function
# py_post()

