########################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../lib.4gm/all.4gs/st_init.4gl  1.10  Delta: 9/4/90
########################################################################

database standard

globals "globals.4gl"

############################################################
function st_init()
############################################################
# This function serves the purpose of initializing the
# accounting toolkit.
#   - handles processing for multi-company
#   - handles turning on/off (true/false arg) database
#     transaction processing.
#
    define
        ini_mod char(8),     # module name
        ini_prog char(8),    # program name
        ini_trx smallint,    # use transactions ? (boolean true/false)
        compid char(8),      # company identification
        dbname char(14),     # database name for that company id
        logpath char(64),    # pathname of that database's logfile
        n smallint           # working number

    # This may look funny, but it is required so the logo() function
    # will be loaded by the linker at this time.  We need it loaded
    # at this time, but it will be run at a later time.
    if FALSE
    then
        call logo()
    end if

    call get_mod() returning ini_mod, ini_prog

    # default to the standard database
    let compid = ""
    let dbname = ""
    let logpath = ""

    # parse the command line for "-c $company" where company is
    # the company id that is used as a key into stxcompr to find
    # the database name, and the logfile path.
    for n = 1 to num_args() - 1
        if arg_val(n) = "-c"
        then
            let compid = arg_val(n+1)
        end if
    end for

    # try to find the database name in stxcompr
    if compid is not null
    then
        whenever error continue
        select db_name, logfile into  dbname, logpath
            from stxcompr where comp_id = compid
        whenever error call error_handler
        if sqlca.sqlcode != 0
        then
            # try opening the database to see if it exists
            whenever error continue
            database compid
            whenever error call error_handler
            if sqlca.sqlcode = 0
              then 
                  let dbname = compid
              else 
                  let sqlca.sqlcode = 0
                  call fg_error("lib_all","st_init", 1) 
                      # Specified company not found - using standard
            end if
        end if
    end if
    if dbname is null then let dbname  = "standard" end if
    if logpath is null or logpath = " "
    then
        let logpath = "../../data/logfile"
    end if

    close database
    database dbname

# to keep nrows in systables as current as possible, update statistics
# every time a posting program is run.
    if ini_prog[1,2] = "p_"
    then
        update statistics
    end if

# if the current sql engine is the old (.03) standard engine, then
# check if logging should be started.
    if log_change_ok()
    then
        whenever error continue
        select use_trx into ini_trx from stxprogr where
            module = ini_mod and progname = ini_prog
        whenever error call error_handler
    
        if sqlca.sqlcode != 0
        then
            # set default trx_processing to true if post type process...
            let sqlca.sqlcode = 0
            if ini_prog[1,2] = "i_" or ini_prog[1,2] = "o_"
            then
                let ini_trx = false
            end if
            if ini_prog[1,2] = "p_"
            then
                let ini_trx = true
            end if
        end if
    
        if ini_trx = true
          then call add_log(dbname,logpath)
          else call chk_log(dbname,logpath)
        end if
    end if

    # initialize some cursors up-front
    call al_addlinit()

    options help key control-q

    return
end function
# st_init()

############################################################
function add_log(dbname,logpath)
############################################################
# Add a record "syslog" into systables.
# Close and reopen the database for transaction logging.
# Deletes the record "syslog"
#
    define
        log_rowid integer,
        dbname char(14),
        logpath char(64)

    whenever error continue
    select rowid into log_rowid from systables where tabname = "syslog"
    if sqlca.sqlcode != 0
    then
        insert into systables
            (tabname,owner,dirpath,tabid,rowsize,ncols,
            nindexes,nrows,created,version,tabtype)
        values
            ("syslog","informix",logpath,
              0,0,1,0,0,today,0,"L")
        if sqlca.sqlcode != 0
        then
            # Cannot add the syslog row into systables
            let sqlca.sqlcode = 0
            call fg_error("lib_all","add_log",1) 
            exit program(1)
        end if
        close database
        database dbname
        if sqlca.sqlcode != 0
        then
            # Cannot open the database logfile.
            let sqlca.sqlcode = 0
            call fg_error("lib_all","add_log",2) 
            exit program(1)
        end if
        delete from systables where tabname = "syslog"
    end if
    whenever error call error_handler

end function
# add_log()

############################################################
function chk_log(dbname,logpath)
############################################################
# Check the table systables to see if transaction is set on.
# If transaction is on, then it sleeps until the transaction
# record is removed.
#
    define
        sys_rowid integer,
        dbname char(14),
        logpath char(64),
        cnt smallint

        whenever error continue
        select rowid into sys_rowid from systables
            where tabname = "syslog"
        if sqlca.sqlcode = notfound   #database was open without transaction
        then
            let sqlca.sqlcode = 0
        else                   #waits for database to open without trx
            open window wait_sys at 2,4 with 1 rows, 60 columns
                attribute(white,border)
            display " Waiting for system table to become available..."
            at 1,1
            let cnt = 0
            while cnt < 5
                sleep 5
                select rowid into sys_rowid from systables
                    where tabname = "syslog"
                if sqlca.sqlcode = 0
                then
                    let cnt = cnt + 1
                    continue while
                else
                    let sqlca.sqlcode = 0
                    exit while
                end if
            end while
            if cnt = 5
            then
                # problem with syslog record.  it should never be there
                # for this long.  the only time the syslog record should
                # be there is when you run a process with transactions,
                # and in that case, the record is inserted only for the
                # time it takes to close the database, re-open it (with
                # transactions), then delete that syslog record.
                # were going to have to 86 that syslog record...

                delete from systables where rowid = sys_rowid
            end if

            close window wait_sys
            close database
            database dbname
        end if
        whenever error call error_handler
end function
# chk_log()


########################################################################
function log_change_ok()
# determines if it is possible to change transaction logging
########################################################################
# returns true if ok to change transaction logging, false if not.
# also ensures logging is on for 4.0 standard engine

# check for "dirpath".  if not found, then turbo/on-line so return false
    select rowid from syscolumns where tabid = 1 and colname = "dirpath"
    if sqlca.sqlcode = notfound 
    then 
        let sqlca.sqlcode = 0
        return false 
    end if

# check for "text".  if found, then pre-4.0, so return true
    select rowid from syscolumns where tabid = 6 and colname = "text"
    if sqlca.sqlcode = 0 
    then 
        let sqlca.sqlcode = 0
        return true 
    end if

# check for the existance of a log.  if not there, explain the problem
# and then exit the program.
    select rowid from systables where tabname = "syslog"
    if sqlca.sqlcode = notfound
    then
        let sqlca.sqlcode = 0
        call fg_error("lib_all","st_init",2) # must be a log for inx 4.0
        exit program (1)
    end if

# since the log exists for inx 4.0, return false
    return false
end function
# log_change_ok()

