########################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../lib.4gm/all.4gs/trx_post.4gl  1.5  Delta: 9/4/90
########################################################################

database standard

define
    l_prepared smallint

#####################################################################
function trx_post(post_or_check, orig_journal, doc_no,
                 post_no, post_date, doc_date, ref_code, doc_desc)
# returns an integer status variable (0=pass, non-0=fail)
#####################################################################
#  This routine either posts to or checks the stxtranr table for dups.
#  It is designed to be run in "CHECK" mode during the edit list phase,
#  and in "POST" mode during the posting phase.
#  It is designed to be run from the ar_post, gl_post, ap_post, etc.
#  library routines.
#  It is called once for each document (doc_no)
#
#  Data elements passed:
#    post_or_check char(5),  "POST" or "CHECK"  tells ar_post to check
#                              the posting (for edit lists), or post it.
#    orig_journal char(2),   original journal (AR, OE, etc...).
#    doc_no integer,         document number (sequential and congruent 
#                              for each document within a journal).
#    post_no integer,        posting number (post only).
#    post_date date,         posting date (post only).
#    doc_date date,          document entry date.
#    ref_code char(6),       customer code.
#    doc_desc char(30)       document description.
#
#  Return values:
#         0 = if "CHECK" or if "POST" and posted ok
#         1 = insert failed (post only)
#         2 = non-exact dup found
#
#  What this process does:
#    if "CHECK" then check for duplicate & return error if dup found.
#    if "POST" then insert the row into stxtranr & return error if insert fails.
#
    define
        post_or_check char(5),
        orig_journal char(2),
        doc_no integer,
        post_no integer,
        post_date date,
        doc_date date,
        ref_code char(6),
        doc_desc char(30),
        doc_count integer,
        p_xtranr record like stxtranr.*

    # Don't do anything if just checking.
    if post_or_check = "CHECK" then return 0 end if

    if l_prepared = 123
    then else
        prepare curs_1 from
          "select * from stxtranr where orig_journal = ? and doc_no = ?"
        declare get_xtranr cursor for curs_1
        prepare xtranr_insert from
          "insert into stxtranr values(?,?,?,?,?,?,?)"
        let l_prepared = 123
    end if

    # See if the transaction exists
    whenever error continue
    open get_xtranr using orig_journal, doc_no
    fetch get_xtranr into p_xtranr.*
    whenever error call error_handler

    case sqlca.sqlcode
      when 0
        if
          p_xtranr.post_no != post_no or
          p_xtranr.post_date != post_date or
          p_xtranr.doc_date != doc_date or
          p_xtranr.ref_code != ref_code or
          p_xtranr.doc_desc != doc_desc
        then
            # this isn't the same one as posted earlier - it's a dup...
            return 2
        end if

      when notfound
        # insert the new stxtranr row
        whenever error continue
        execute xtranr_insert using
          orig_journal, doc_no, post_no,
          post_date, doc_date, ref_code, doc_desc
        whenever error call error_handler
        if sqlca.sqlcode != 0 
        then 
            let sqlca.sqlcode = 0
            return 1 
        end if

      otherwise
        # the sql query failed. return as if a dup was found (fail)
        let sqlca.sqlcode = 0
        return 2
    end case

    # everything went ok
    return 0
end function
# trx_post()
